###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###
# -*- coding: utf-8 -*-
""" Server Info Command for rdmc """
import re
import sys
from collections import OrderedDict

import jsonpath_rw

try:
    from rdmc_helper import (
        UI,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        UI,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        ReturnCodes,
    )


class ServerInfoCommand:
    """Show details of a server"""

    def __init__(self):
        self.ident = {
            "name": "serverinfo",
            "usage": None,
            "description": "Shows all information.\n\tExample: serverinfo\n\t\t"
            "serverinfo --all\n\n\t"
            "Show enabled fan, processor, and thermal information.\n\texample: "
            "serverinfo --fans --processors --thermals --proxy\n\n\tShow all memory "
            "and fan information, including absent locations in json format.\n\t"
            "example: serverinfo --proxy --firmware --software --memory --fans --showabsent -j\n",
            "summary": "Shows aggregate health status and details of the currently logged in server.",
            "aliases": ["health", "serverstatus", "systeminfo"],
            "auxcommands": [],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main serverinfo function.

        :param line: string of arguments passed in
        :type line: str.
        :param help_disp: display help flag
        :type line: bool.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        if args:
            raise InvalidCommandLineError("serverinfo command takes no " "arguments.")

        self.serverinfovalidation(options)

        self.optionsvalidation(options)

        info = self.gatherinfo(options)

        if not info:
            raise InvalidCommandLineError("Please verify the commands entered " "and try again.")
        if "proxy" in info and info["proxy"]:
            if "WebProxyConfiguration" in info["proxy"]["Oem"]["Hpe"]:
                info["proxy"] = info["proxy"]["Oem"]["Hpe"]["WebProxyConfiguration"]

        if options.json:
            if options.processors and options.json and self.rdmc.app.typepath.defs.isgen9:
                pass
            elif options.memory and self.rdmc.app.typepath.defs.isgen9:
                pass
            else:
                self.build_json_out(info, options.showabsent)
        else:
            self.prettyprintinfo(info, options.showabsent)

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def gatherinfo(self, options):
        """Gather info to printout based on options

        :param options: command line options
        :type options: list.
        """
        info = {}
        path = self.rdmc.app.typepath.defs.systempath
        if path == "/rest/v1/Systems/1" or "/redfish/v1/Systems/1/":
            path = "/redfish/v1/Systems/"
        else:
            pass
        if options.system:
            info["system"] = OrderedDict()
            csysresults = self.rdmc.app.get_handler(path, service=True, silent=True).dict
            # csysresults = self.rdmc.app.select(selector="ComputerSystemCollection")
            # csysresults = csysresults[0].dict
            members = csysresults["Members"]
            for id in members:
                for mem_id in id.values():
                    if path in mem_id:
                        csysresults = self.rdmc.app.get_handler(mem_id, service=True, silent=True).dict
                try:
                    csysresults = csysresults[0].dict
                except:
                    pass
                if csysresults:
                    info["system"]["Model"] = "%s %s" % (
                        csysresults["Manufacturer"],
                        csysresults["Model"],
                    )
                    info["system"]["Bios Version"] = csysresults["BiosVersion"]

                biosresults = self.rdmc.app.select(selector=self.rdmc.app.typepath.defs.biostype)

                try:
                    biosresults = biosresults[0].dict
                except:
                    pass

                if biosresults:
                    try:
                        info["system"]["Serial Number"] = biosresults["Attributes"]["SerialNumber"]
                    except:
                        if "SerialNumber" in biosresults:
                            info["system"]["Serial Number"] = biosresults["SerialNumber"]
                ethresults = None
                getloc = self.rdmc.app.getidbytype("EthernetInterfaceCollection.")
                if getloc:
                    for loc in getloc:
                        if "/systems/1/" in loc.lower():
                            ethresults = self.rdmc.app.getcollectionmembers(loc)
                            break
                    if ethresults:
                        niccount = 0
                        info["system"]["ethernet"] = OrderedDict()
                        for eth in ethresults:
                            niccount += 1
                            if eth["Name"] == "":
                                if "MACAddress" in eth:
                                    info["system"]["ethernet"][niccount] = eth["MACAddress"]
                                else:
                                    info["system"]["ethernet"][niccount] = eth["PermanentMACAddress"]
                            elif eth["Name"] != "":
                                if "MACAddress" in eth:
                                    info["system"]["ethernet"][eth["Name"]] = eth["MACAddress"]
                                else:
                                    info["system"]["ethernet"][eth["Name"]] = eth["PermanentMACAddress"]
                        info["system"]["NICCount"] = niccount
        if options.thermals or options.fans:
            data = None
            if not self.rdmc.app.typepath.defs.isgen9:
                getloc = self.rdmc.app.getidbytype("Thermal.")
            else:
                getloc = self.rdmc.app.getidbytype("ThermalMetrics.")
            if getloc:
                data = self.rdmc.app.get_handler(getloc[0], silent=True, service=True)
            if options.thermals:
                if not getloc:
                    info["thermals"] = None
                else:
                    info["thermals"] = data.dict["Temperatures"]
            if options.fans:
                if not getloc:
                    info["fans"] = None
                else:
                    info["fans"] = data.dict["Fans"]
        if options.memory:
            data = None
            if self.rdmc.app.typepath.defs.isgen9:
                mem_path = "/redfish/v1/Systems/1/Memory/" + "?$expand=."
                getloc = self.rdmc.app.get_handler(mem_path, silent=True, service=True).dict
                collectiondata = getloc["Oem"][self.rdmc.app.typepath.defs.oemhp]
                if collectiondata and not options.json:
                    sys.stdout.write("---------------------------------\n")
                    sys.stdout.write("Memory/DIMM Board Information:\n")
                    sys.stdout.write("---------------------------------\n")
                    memory_status = "Advanced Memory Protection Status: %s \n" % collectiondata["AmpModeStatus"]
                    sys.stdout.write(memory_status)
                    sys.stdout.write("AmpModeActive: %s \n" % collectiondata["AmpModeActive"])
                    sys.stdout.write("AmpModeSupported: %s \n" % collectiondata["AmpModeSupported"])
                    sys.stdout.write("Type: %s \n" % collectiondata["Type"])
                if collectiondata and options.json:
                    tmp = dict()
                    tmp["Memory"] = dict()
                    tmp["Memory"]["Advanced Memory Protection Status"] = collectiondata["AmpModeStatus"]
                    tmp["Memory"]["AmpModeActive"] = collectiondata["AmpModeActive"]
                    tmp["Memory"]["AmpModeSupported"] = collectiondata["AmpModeSupported"]
                    tmp["Memory"]["Type"] = collectiondata["Type"]
                    UI().print_out_json(tmp)
            getloc = self.rdmc.app.getidbytype("MemoryCollection.")
            if getloc:
                data = self.rdmc.app.getcollectionmembers(getloc[0], fullresp=True)[0]
                info["memory"] = data
            else:
                info["memory"] = None
        if options.proxy:
            data = None
            getloc = self.rdmc.app.getidbytype("NetworkProtocol.")
            if getloc:
                data = self.rdmc.app.getcollectionmembers(getloc[0], fullresp=True)[0]
                info["proxy"] = data
            else:
                info["proxy"] = None
        if options.processors:
            data = None
            if not self.rdmc.app.typepath.defs.isgen9:
                getloc = self.rdmc.app.getidbytype("ProcessorCollection.")
            else:
                # getloc = self.rdmc.app.getidbytype("ProcessorCollection.")
                getloc = self.rdmc.app.getidbytype("Processor.")
                tot_proc = []
                if getloc == "/rest/v1/Systems/1/Processors/1" or "/rest/v1/Systems/1/Processors/2":
                    getloc = "/redfish/v1/Systems/1/Processors/"
                    get_p = self.rdmc.app.get_handler(getloc, service=True, silent=True).dict["Members"]
                    output = ""
                    if get_p and not options.json:
                        sys.stdout.write("------------------------------------------------\n")
                        sys.stdout.write("Processor:\n")
                        sys.stdout.write("------------------------------------------------\n")
                        for pp in get_p:
                            dd = pp["@odata.id"]
                            data = self.rdmc.app.get_handler(dd, service=True, silent=True).dict
                            output += "Processor %s:\n" % data["Id"]
                            output += "\tModel: %s\n" % data["Model"]
                            output += "\tStep: %s\n" % data["ProcessorId"]["Step"]
                            output += "\tSocket: %s\n" % data["Socket"]
                            output += "\tMax Speed: %s MHz\n" % data["MaxSpeedMHz"]
                            try:
                                output += (
                                    "\tSpeed: %s MHz\n"
                                    % data["Oem"][self.rdmc.app.typepath.defs.oemhp]["RatedSpeedMHz"]
                                )
                            except KeyError:
                                pass
                            output += "\tCores: %s\n" % data["TotalCores"]
                            output += "\tThreads: %s\n" % data["TotalThreads"]
                            try:
                                for cache in data["Oem"][self.rdmc.app.typepath.defs.oemhp]["Cache"]:
                                    output += "\t%s: %s KB\n" % (
                                        cache["Name"],
                                        cache["InstalledSizeKB"],
                                    )
                            except KeyError:
                                pass
                            try:
                                output += "\tHealth: %s\n" % data["Status"]["Health"]
                            except KeyError:
                                pass
                        self.rdmc.ui.printer(output, verbose_override=True)
                    if get_p and options.json:
                        for pp in get_p:
                            dd = pp["@odata.id"]
                            data = self.rdmc.app.get_handler(dd, service=True, silent=True).dict
                            process = "Processor %s" % data["Id"]
                            tmp = dict()
                            tmp[process] = dict()
                            tmp[process]["Processor"] = data["Id"]
                            tmp[process]["Model"] = data["Model"]
                            tmp[process]["Step"] = data["ProcessorId"]["Step"]
                            tmp[process]["Socket"] = data["Socket"]
                            tmp[process]["Max Speed"] = data["MaxSpeedMHz"]
                            try:
                                tmp[process].update(
                                    {
                                        "Speed": "%s MHz"
                                        % data["Oem"][self.rdmc.app.typepath.defs.oemhp]["RatedSpeedMHz"]
                                    }
                                )
                            except KeyError:
                                pass
                            tmp[process].update({"Cores": data["TotalCores"]})
                            tmp[process].update({"Threads": data["TotalThreads"]})
                            UI().print_out_json(tmp)
                else:
                    pass
            if getloc and not self.rdmc.app.typepath.defs.isgen9:
                data = self.rdmc.app.getcollectionmembers(getloc[0])

                info["processor"] = data
            else:
                info["processor"] = None
        if options.power:
            data = None
            if not self.rdmc.app.typepath.defs.isgen9:
                getloc = self.rdmc.app.getidbytype("Power.")
            else:
                getloc = self.rdmc.app.getidbytype("PowerMetrics.")
            if getloc:
                data = self.rdmc.app.get_handler(getloc[0], silent=True, service=True)
                info["power"] = data.dict
            else:
                info["power"] = None
        if options.firmware:
            data = None
            if not self.rdmc.app.typepath.defs.isgen10:
                getloc = self.rdmc.app.getidbytype("FwSwVersionInventory")
            else:
                getloc = self.rdmc.app.getidbytype("SoftwareInventoryCollection.")

            for gloc in getloc:
                if "FirmwareInventory" in gloc:
                    data = self.rdmc.app.getcollectionmembers(gloc)
                    # if not self.rdmc.app.typepath.defs.isgen10:
                    #    data1 = list(data)
            info["firmware"] = data
        if options.software:
            data = None
            if not self.rdmc.app.typepath.defs.isgen10:
                getloc = self.rdmc.app.getidbytype("SoftwareInventory")
            else:
                getloc = self.rdmc.app.getidbytype("SoftwareInventoryCollection.")
            data_list = list()
            for gloc in getloc:
                if "SoftwareInventory" in gloc:
                    if not self.rdmc.app.typepath.defs.isgen10:
                        data = self.rdmc.app.getcollectionmembers(gloc)
                        data_list.append(data.dict)
                    else:
                        data = self.rdmc.app.getcollectionmembers(gloc)
            if not self.rdmc.app.typepath.defs.isgen10:
                info["software"] = data_list
            else:
                info["software"] = data
        if not options.showabsent:
            jsonpath_expr = jsonpath_rw.parse("$..State")
            matches = jsonpath_expr.find(info)
            matches.reverse()

            for match in matches:
                if match.value.lower() == "absent":
                    arr = None
                    statepath = "/" + str(match.full_path).replace(".", "/")
                    arr = re.split("[[]]", statepath)
                    if arr:
                        removedict = None
                        start = arr[0].split("/")
                        for key in start:
                            if key:
                                if not removedict:
                                    removedict = info[key]
                                else:
                                    removedict = removedict[key]
                        del removedict[int(arr[1])]
        return info

    def build_json_out(self, info, absent):
        headers = list(info.keys())
        content = dict()
        if "power" in headers and info["power"]:
            data = info["power"]
            if data is not None:
                for control in data["PowerControl"]:
                    if "PowerCapacityWatts" in control:
                        power_cap = {"Total Power Capacity": "%s W" % control["PowerCapacityWatts"]}
                    if "PowerConsumedWatts" in control:
                        power_cap.update({"Total Power Consumed": "%s W" % control["PowerConsumedWatts"]})
                    if "PowerMetrics" in control:
                        if "AverageConsumedWatts" in control["PowerMetrics"]:
                            power_mertic = {"Average Power": "%s W" % control["PowerMetrics"]["AverageConsumedWatts"]}
                        if "MaxConsumedWatts" in control["PowerMetrics"]:
                            power_mertic.update(
                                {"Max Consumed Power": "%s W" % control["PowerMetrics"]["MaxConsumedWatts"]}
                            )
                        if "MinConsumedWatts" in control["PowerMetrics"]:
                            power_mertic.update(
                                {"Minimum Consumed Power": "%s W" % control["PowerMetrics"]["MinConsumedWatts"]}
                            )
                        if "IntervalInMin" in control["PowerMetrics"]:
                            powercontent = (
                                "Power Metrics on %s min. Intervals" % control["PowerMetrics"]["IntervalInMin"]
                            )
                    test = {powercontent: power_mertic}
                    content = {"power": power_cap}
                    content["power"].update(test)
                try:
                    powersuplies = []
                    for supply in data["PowerSupplies"]:
                        power_supply = "Power Supply %s" % supply["Oem"][self.rdmc.app.typepath.defs.oemhp]["BayNumber"]
                        powersupply = {}
                        if "PowerCapacityWatts" in supply:
                            powersupply = {"Power Capacity": "%s W" % supply["PowerCapacityWatts"]}
                        if "LastPowerOutputWatts" in supply:
                            powersupply.update({"Last Power Output": "%s W" % supply["LastPowerOutputWatts"]})
                        if "LineInputVoltage" in supply:
                            powersupply.update({"Input Voltage": "%s V" % supply["LineInputVoltage"]})
                        if "LineInputVoltageType" in supply:
                            powersupply.update({"Input Voltage Type": supply["LineInputVoltageType"]})
                        if "HotplugCapable" in supply["Oem"][self.rdmc.app.typepath.defs.oemhp]:
                            powersupply.update(
                                {"Hotplug Capable": supply["Oem"][self.rdmc.app.typepath.defs.oemhp]["HotplugCapable"]}
                            )
                        if "iPDUCapable" in supply["Oem"][self.rdmc.app.typepath.defs.oemhp]:
                            powersupply.update(
                                {"iPDU Capable": supply["Oem"][self.rdmc.app.typepath.defs.oemhp]["iPDUCapable"]}
                            )
                        try:
                            if "Health" in supply["Status"]:
                                powersupply.update({"Health": supply["Status"]["Health"]})
                        except KeyError:
                            pass
                        # if absent:
                        try:
                            if "State" in supply["Status"]:
                                powersupply.update({"State": supply["Status"]["State"]})
                        except KeyError:
                            pass
                        # powerdetails = {power_supply: powersupply}
                        powersuplies.append(powersupply)
                        content["power"].update({"PowerSupplies": powersuplies})
                    for redundancy in data["Redundancy"]:
                        redund_name = redundancy["Name"]
                        redundancy_dict = {"Redundancy Mode": redundancy["Mode"]}
                        try:
                            redundancy_dict.update({"Redundancy Health": redundancy["Status"]["Health"]})
                            redundancy_dict.update({"Redundancy State": redundancy["Status"]["State"]})
                            content.update({redund_name: redundancy_dict})
                        except KeyError:
                            pass
                except KeyError:
                    pass

        if "firmware" in headers and info["firmware"]:
            firmware_info = []
            data = info["firmware"]
            if data is not None:
                if not self.rdmc.app.typepath.defs.isgen10:
                    for key, fw in data.items():
                        for fw_gen9 in fw:
                            fw_str = fw_gen9["Name"] + ": " + fw_gen9["VersionString"]
                            firmware_info.append(fw_str)
                else:
                    for fw in data:
                        fw_str = fw["Name"] + ": " + fw["Version"]
                        firmware_info.append(fw_str)
            content.update({"firmware": firmware_info})

        if "software" in headers and info["software"]:
            output = ""
            software_info = {}
            data = info["software"]
            if data is not None:
                if isinstance(data, dict) or isinstance(data, list):
                    for sw in data:
                        software_info.update({sw["Name"]: sw["Version"]})
                else:
                    # if not options.json:
                    software_info = "No information available for the server\n"
                    self.rdmc.ui.printer(software_info, verbose_override=True)
            content.update({"software": software_info})

        if "memory" in headers and info["memory"]:
            data = info["memory"]
            count = 1
            if data is not None:
                collectiondata = data["Oem"][self.rdmc.app.typepath.defs.oemhp]
                output = "Memory/DIMM Board Information"

                memory_status = "Advanced Memory Protection Status: %s" % collectiondata["AmpModeStatus"]
                memorylist = {}
                for board in collectiondata["MemoryList"]:
                    board_detail = "Board CPU: %s" % board["BoardCpuNumber"]
                    memory_info = {"Total Memory Size": "%s MiB" % board["BoardTotalMemorySize"]}
                    memory_info.update({"Board Memory Frequency": "%s MHz" % board["BoardOperationalFrequency"]})
                    memory_info.update({"Board Memory Voltage": "%s MiB" % board["BoardOperationalVoltage"]})
                    memorylist.update({board_detail: memory_info})
                memorystatus = {memory_status: memorylist}
                memoryinfo = {output: memorystatus}

                for dimm in data[self.rdmc.app.typepath.defs.collectionstring]:
                    memoryconfig = {"Location": dimm["DeviceLocator"]}
                    try:
                        memoryconfig.update(
                            {
                                "Memory Type": "%s %s"
                                % (
                                    dimm["MemoryType"],
                                    dimm["MemoryDeviceType"],
                                )
                            }
                        )
                    except KeyError:
                        memoryconfig.update({"Memory Type": dimm["MemoryType"]})
                    memoryconfig.update({"Capacity": "%s MiB" % dimm["CapacityMiB"]})
                    try:
                        memoryconfig.update({"Speed": "%s MHz" % dimm["OperatingSpeedMhz"]})
                        memoryconfig.update({"Status": dimm["Oem"][self.rdmc.app.typepath.defs.oemhp]["DIMMStatus"]})
                        memoryconfig.update({"Health": dimm["Status"]["Health"]})
                    except KeyError:
                        pass

                    if absent:
                        try:
                            memoryconfig.update({"State": dimm["Status"]["State"]})
                        except KeyError:
                            pass

                    memory_config = "Memory/DIMM Configuration" + " " + str(count)
                    memoryinfo.update({memory_config: memoryconfig})
                    content.update({"memory": memoryinfo})
                    count = count + 1

        if "fans" in headers and info["fans"]:
            fan_output = {}
            fan_details = {}
            if info["fans"] is not None:
                for fan in info["fans"]:
                    fan_name = ""
                    if not self.rdmc.app.typepath.defs.isgen9:
                        fan_name = "%s" % fan["Name"]
                    else:
                        fan_name = "%s" % fan["FanName"]
                    fan_output.update({"Location": fan["Oem"][self.rdmc.app.typepath.defs.oemhp]["Location"]})
                    if "Reading" in fan:
                        fan_output.update({"Reading": "%s%%" % fan["Reading"]})
                        fan_output.update({"Redundant": fan["Oem"][self.rdmc.app.typepath.defs.oemhp]["Redundant"]})
                        fan_output.update(
                            {"Hot Pluggable": fan["Oem"][self.rdmc.app.typepath.defs.oemhp]["HotPluggable"]}
                        )
                    try:
                        if "Health" in fan["Status"]:
                            fan_output.update({"Health": fan["Status"]["Health"]})
                    except KeyError:
                        pass

                    # if absent:
                    try:
                        fan_output.update({"State": fan["Status"]["State"]})
                    except KeyError:
                        pass
                    fan_details.update({fan_name: fan_output})
                    content.update({"fans": fan_details})

        if "thermals" in headers and info["thermals"]:
            if info["thermals"] is not None:
                sensor = ""
                thermal_detail = {}
                for temp in info["thermals"]:
                    if "SensorNumber" in temp:
                        sensor = "Sensor #%s:" % temp["SensorNumber"]
                    if "PhysicalContext" in temp:
                        thermal_info = {"Location": temp["PhysicalContext"]}
                    thermal_info.update({"Current Temp": "%s C" % temp["ReadingCelsius"]})
                    if "UpperThresholdCritical" in temp:
                        thermal_info.update({"Critical Threshold": "%s C" % temp["UpperThresholdCritical"]})
                    else:
                        thermal_info.update({"Critical Threshold": "-"})
                    if "UpperThresholdFatal" in temp:
                        thermal_info.update({"Fatal Threshold": "%s C" % temp["UpperThresholdFatal"]})
                    else:
                        thermal_info.update({"Fatal Threshold": "-"})
                    try:
                        if "Health" in temp["Status"]:
                            thermal_info.update({"Health": temp["Status"]["Health"]})
                    except KeyError:
                        pass
                    if absent:
                        try:
                            thermal_info.update({"State": temp["Status"]["State"]})
                        except KeyError:
                            pass
                    thermal_detail.update({sensor: thermal_info})
                    content.update({"thermals": thermal_detail})

        if "processor" in headers and info["processor"]:
            data = info["processor"]
            processor_info = {}
            if data is not None:
                if not self.rdmc.app.typepath.defs.isgen9:
                    for processor in data:
                        process = "Processor %s" % processor["Id"]
                        processor_date = {"Model": processor["Model"]}
                        processor_date.update({"Step": processor["ProcessorId"]["Step"]})
                        processor_date.update({"Socket": processor["Socket"]})
                        processor_date.update({"Max Speed": "%s MHz" % processor["MaxSpeedMHz"]})
                        try:
                            processor_date.update(
                                {
                                    "Speed": "%s MHz"
                                    % processor["Oem"][self.rdmc.app.typepath.defs.oemhp]["RatedSpeedMHz"]
                                }
                            )
                        except KeyError:
                            pass
                        processor_date.update({"Cores": processor["TotalCores"]})
                        processor_date.update({"Threads": processor["TotalThreads"]})
                        try:
                            for cache in processor["Oem"][self.rdmc.app.typepath.defs.oemhp]["Cache"]:
                                processor_date.update({cache["Name"]: "%s KB" % cache["InstalledSizeKB"]})
                        except KeyError:
                            pass
                        try:
                            processor_date.update({"Health": processor["Status"]["Health"]})
                        except KeyError:
                            pass
                        if absent:
                            try:
                                processor_date.update({"State": processor["Status"]["State"]})
                            except KeyError:
                                pass
                        processor_info.update({process: processor_date})
                else:
                    data = data.dict
                    # for processor in data:
                    process = "Processor %s" % data["Id"]
                    processor_date = {"Model": data["Model"]}
                    processor_date.update({"Step": data["ProcessorId"]["Step"]})
                    processor_date.update({"Socket": data["Socket"]})
                    processor_date.update({"Max Speed": "%s MHz" % data["MaxSpeedMHz"]})
                    try:
                        processor_date.update(
                            {"Speed": "%s MHz" % data["Oem"][self.rdmc.app.typepath.defs.oemhp]["RatedSpeedMHz"]}
                        )
                    except KeyError:
                        pass
                    processor_date.update({"Cores": data["TotalCores"]})
                    processor_date.update({"Threads": data["TotalThreads"]})
                    try:
                        for cache in data["Oem"][self.rdmc.app.typepath.defs.oemhp]["Cache"]:
                            processor_date.update({cache["Name"]: "%s KB" % cache["InstalledSizeKB"]})
                    except KeyError:
                        pass
                    try:
                        processor_date.update({"Health": data["Status"]["Health"]})
                    except KeyError:
                        pass
                    if absent:
                        try:
                            processor_date.update({"State": data["Status"]["State"]})
                        except KeyError:
                            pass
                    processor_info.update({process: processor_date})
                content.update({"processor": processor_info})

        if "proxy" in headers and info["proxy"]:
            proxy_info = {}
            data = info["proxy"]
            try:
                if data is not None:
                    for k, v in data.items():
                        proxy_info.update({k: v})
            except KeyError:
                pass
            content.update({"proxy": proxy_info})

        if "system" in headers:
            data = info["system"]
            system = {}
            if data is not None:
                for key, val in list(data.items()):
                    if key == "ethernet":
                        embedded_nic = {"Embedded NIC Count": data["NICCount"]}
                        system.update(embedded_nic)
                        for name in sorted(data["ethernet"]):
                            mac_name = str(name) + " " + "MAC"
                            mac = {mac_name: data["ethernet"][name]}
                            system.update(mac)
                    elif not key == "NICCount":
                        nic = {key: val}
                        system.update(nic)
            content.update({"system": system})

        UI().print_out_json(content)

    def prettyprintinfo(self, info, absent):
        """Print info in human readable form from json

        :param info: info data collected
        :type info: dict.
        :param absent: flag to show or hide absent components
        :type absent: bool.
        """
        output = ""
        headers = list(info.keys())
        if "system" in headers:
            data = info["system"]
            output = "------------------------------------------------\n"
            output += "System:\n"
            output += "------------------------------------------------\n"
            if data is not None:
                for key, val in list(data.items()):
                    if key == "ethernet":
                        output += "Embedded NIC Count: %s\n" % data["NICCount"]
                        for name in sorted(data["ethernet"]):
                            output += "%s MAC: %s\n" % (name, data["ethernet"][name])
                    elif not key == "NICCount":
                        output += "%s: %s\n" % (key, val)
            self.rdmc.ui.printer(output, verbose_override=True)

        if "firmware" in headers and info["firmware"]:
            data = info["firmware"]
            output = "------------------------------------------------\n"
            output += "Firmware: \n"
            output += "------------------------------------------------\n"
            if data is not None:
                if not self.rdmc.app.typepath.defs.isgen10:
                    for key, fw in data.items():
                        for fw_gen9 in fw:
                            output += "%s : %s\n" % (
                                fw_gen9["Name"],
                                fw_gen9["VersionString"],
                            )
                else:
                    for fw in data:
                        output += "%s : %s\n" % (fw["Name"], fw["Version"])
            self.rdmc.ui.printer(output, verbose_override=True)

        if "software" in headers and info["software"]:
            output = ""
            data = info["software"]
            if not isinstance(data, dict) and not isinstance(data, list):
                data = data.dict
            output = "------------------------------------------------\n"
            output += "Software: \n"
            output += "------------------------------------------------\n"
            if data is not None:
                if isinstance(data, dict) or isinstance(data, list):
                    for sw in data:
                        output += "%s : %s\n" % (sw["Name"], sw["Version"])
                else:
                    output = "No information available for the server\n"
            self.rdmc.ui.printer(output, verbose_override=True)

        if "proxy" in headers and info["proxy"]:
            output = ""
            data = info["proxy"]
            output = "------------------------------------------------\n"
            output += "Proxy: \n"
            output += "------------------------------------------------\n"
            try:
                if data is not None:
                    for k, v in data.items():
                        output += "%s : %s\n" % (k, v)
            except KeyError:
                pass
            self.rdmc.ui.printer(output, verbose_override=True)

        if "processor" in headers and info["processor"]:
            output = ""
            data = info["processor"]
            output = "------------------------------------------------\n"
            output += "Processor:\n"
            output += "------------------------------------------------\n"
            if data is not None:
                if not self.rdmc.app.typepath.defs.isgen9:
                    for processor in data:
                        output += "Processor %s:\n" % processor["Id"]
                        output += "\tModel: %s\n" % processor["Model"]
                        output += "\tStep: %s\n" % processor["ProcessorId"]["Step"]
                        output += "\tSocket: %s\n" % processor["Socket"]
                        output += "\tMax Speed: %s MHz\n" % processor["MaxSpeedMHz"]
                        try:
                            output += (
                                "\tSpeed: %s MHz\n"
                                % processor["Oem"][self.rdmc.app.typepath.defs.oemhp]["RatedSpeedMHz"]
                            )
                        except KeyError:
                            pass
                        output += "\tCores: %s\n" % processor["TotalCores"]
                        output += "\tThreads: %s\n" % processor["TotalThreads"]
                        try:
                            for cache in processor["Oem"][self.rdmc.app.typepath.defs.oemhp]["Cache"]:
                                output += "\t%s: %s KB\n" % (
                                    cache["Name"],
                                    cache["InstalledSizeKB"],
                                )
                        except KeyError:
                            pass
                        try:
                            output += "\tHealth: %s\n" % processor["Status"]["Health"]
                        except KeyError:
                            pass
                        if absent:
                            try:
                                output += "\tState: %s\n" % processor["Status"]["State"]
                            except KeyError:
                                pass
                    self.rdmc.ui.printer(output, verbose_override=True)
                else:
                    data = data.dict
                    output += "Processor %s:\n" % data["Id"]
                    output += "\tModel: %s\n" % data["Model"]
                    output += "\tStep: %s\n" % data["ProcessorId"]["Step"]
                    output += "\tSocket: %s\n" % data["Socket"]
                    output += "\tMax Speed: %s MHz\n" % data["MaxSpeedMHz"]
                    try:
                        output += "\tSpeed: %s MHz\n" % data["Oem"][self.rdmc.app.typepath.defs.oemhp]["RatedSpeedMHz"]
                    except KeyError:
                        pass
                    output += "\tCores: %s\n" % data["TotalCores"]
                    output += "\tThreads: %s\n" % data["TotalThreads"]
                    try:
                        for cache in data["Oem"][self.rdmc.app.typepath.defs.oemhp]["Cache"]:
                            output += "\t%s: %s KB\n" % (
                                cache["Name"],
                                cache["InstalledSizeKB"],
                            )
                    except KeyError:
                        pass
                    try:
                        output += "\tHealth: %s\n" % data["Status"]["Health"]
                    except KeyError:
                        pass
                    if absent:
                        try:
                            output += "\tState: %s\n" % data["Status"]["State"]
                        except KeyError:
                            pass
                    self.rdmc.ui.printer(output, verbose_override=True)

        if "memory" in headers and info["memory"]:
            data = info["memory"]
            if data is not None:
                collectiondata = data["Oem"][self.rdmc.app.typepath.defs.oemhp]
                output = "------------------------------------------------\n"
                output += "Memory/DIMM Board Information:\n"
                output += "------------------------------------------------\n"
                output += "Advanced Memory Protection Status: %s\n" % collectiondata["AmpModeStatus"]
                for board in collectiondata["MemoryList"]:
                    output += "Board CPU: %s \n" % board["BoardCpuNumber"]
                    output += "\tTotal Memory Size: %s MiB\n" % board["BoardTotalMemorySize"]
                    output += "\tBoard Memory Frequency: %s MHz\n" % board["BoardOperationalFrequency"]
                    output += "\tBoard Memory Voltage: %s MiB\n" % board["BoardOperationalVoltage"]
                output += "------------------------------------------------\n"
                output += "Memory/DIMM Configuration:\n"
                output += "------------------------------------------------\n"
                for dimm in data[self.rdmc.app.typepath.defs.collectionstring]:
                    output += "Location: %s\n" % dimm["DeviceLocator"]
                    try:
                        output += "Memory Type: %s %s\n" % (
                            dimm["MemoryType"],
                            dimm["MemoryDeviceType"],
                        )
                    except KeyError:
                        output += "Memory Type: %s\n" % dimm["MemoryType"]
                    output += "Capacity: %s MiB\n" % dimm["CapacityMiB"]
                    try:
                        output += "Speed: %s MHz\n" % dimm["OperatingSpeedMhz"]

                        output += "Status: %s\n" % dimm["Oem"][self.rdmc.app.typepath.defs.oemhp]["DIMMStatus"]
                        output += "Health: %s\n" % dimm["Status"]["Health"]
                    except KeyError:
                        pass
                    if absent:
                        try:
                            output += "State: %s\n" % dimm["Status"]["State"]
                        except KeyError:
                            pass
                    output += "\n"
            self.rdmc.ui.printer(output, verbose_override=True)

        if "power" in headers and info["power"]:
            data = info["power"]
            output = "------------------------------------------------\n"
            output += "Power:\n"
            output += "------------------------------------------------\n"
            if data is not None:
                for control in data["PowerControl"]:
                    output += "Total Power Capacity: %s W\n" % control["PowerCapacityWatts"]
                    output += "Total Power Consumed: %s W\n" % control["PowerConsumedWatts"]
                    output += "\n"
                    output += "Power Metrics on %s min. Intervals:\n" % control["PowerMetrics"]["IntervalInMin"]
                    output += "\tAverage Power: %s W\n" % control["PowerMetrics"]["AverageConsumedWatts"]
                    output += "\tMax Consumed Power: %s W\n" % control["PowerMetrics"]["MaxConsumedWatts"]
                    output += "\tMinimum Consumed Power: %s W\n" % control["PowerMetrics"]["MinConsumedWatts"]
                try:
                    for supply in data["PowerSupplies"]:
                        output += "------------------------------------------------\n"
                        output += "Power Supply %s:\n" % supply["Oem"][self.rdmc.app.typepath.defs.oemhp]["BayNumber"]
                        output += "------------------------------------------------\n"
                        if "PowerCapacityWatts" in supply:
                            output += "Power Capacity: %s W\n" % supply["PowerCapacityWatts"]
                        if "LastPowerOutputWatts" in supply:
                            output += "Last Power Output: %s W\n" % supply["LastPowerOutputWatts"]
                        if "LineInputVoltage" in supply:
                            output += "Input Voltage: %s V\n" % supply["LineInputVoltage"]
                        if "LineInputVoltageType" in supply:
                            output += "Input Voltage Type: %s\n" % supply["LineInputVoltageType"]
                        if "HotplugCapable" in supply["Oem"][self.rdmc.app.typepath.defs.oemhp]:
                            output += (
                                "Hotplug Capable: %s\n"
                                % supply["Oem"][self.rdmc.app.typepath.defs.oemhp]["HotplugCapable"]
                            )
                        if "iPDUCapable" in supply["Oem"][self.rdmc.app.typepath.defs.oemhp]:
                            output += (
                                "iPDU Capable: %s\n" % supply["Oem"][self.rdmc.app.typepath.defs.oemhp]["iPDUCapable"]
                            )
                        try:
                            if "Health" in supply["Status"]:
                                output += "Health: %s\n" % supply["Status"]["Health"]
                        except KeyError:
                            pass

                        try:
                            if "State" in supply["Status"]:
                                output += "State: %s\n" % supply["Status"]["State"]
                        except KeyError:
                            pass
                    for redundancy in data["Redundancy"]:
                        output += "------------------------------------------------\n"
                        output += "%s\n" % redundancy["Name"]
                        output += "------------------------------------------------\n"
                        output += "Redundancy Mode: %s\n" % redundancy["Mode"]
                        try:
                            output += "Redundancy Health: %s\n" % redundancy["Status"]["Health"]
                            output += "Redundancy State: %s\n" % redundancy["Status"]["State"]
                        except KeyError:
                            pass
                except KeyError:
                    pass
            self.rdmc.ui.printer(output, verbose_override=True)

        if "fans" in headers and info["fans"]:
            output = "------------------------------------------------\n"
            output += "Fan(s):\n"
            output += "------------------------------------------------\n"
            if info["fans"] is not None:
                for fan in info["fans"]:
                    if not self.rdmc.app.typepath.defs.isgen9:
                        output += "%s:\n" % fan["Name"]
                    else:
                        output += "%s:\n" % fan["FanName"]
                    output += "\tLocation: %s\n" % fan["Oem"][self.rdmc.app.typepath.defs.oemhp]["Location"]
                    if "Reading" in fan:
                        output += "\tReading: %s%%\n" % fan["Reading"]
                        output += "\tRedundant: %s\n" % fan["Oem"][self.rdmc.app.typepath.defs.oemhp]["Redundant"]
                        output += (
                            "\tHot Pluggable: %s\n" % fan["Oem"][self.rdmc.app.typepath.defs.oemhp]["HotPluggable"]
                        )
                    try:
                        if "Health" in fan["Status"]:
                            output += "\tHealth: %s\n" % fan["Status"]["Health"]
                    except KeyError:
                        pass

                    try:
                        output += "\tState: %s\n" % fan["Status"]["State"]
                    except KeyError:
                        pass
            self.rdmc.ui.printer(output, verbose_override=True)

        if "thermals" in headers and info["thermals"]:
            output = "------------------------------------------------\n"
            output += "Thermal:\n"
            output += "------------------------------------------------\n"
            if info["thermals"] is not None:
                for temp in info["thermals"]:
                    if "SensorNumber" in temp:
                        output += "Sensor #%s:\n" % temp["SensorNumber"]
                    if "PhysicalContext" in temp:
                        output += "\tLocation: %s\n" % temp["PhysicalContext"]
                    output += "\tCurrent Temp: %s C\n" % temp["ReadingCelsius"]
                    if "UpperThresholdCritical" in temp:
                        output += "\tCritical Threshold: %s C\n" % temp["UpperThresholdCritical"]
                    else:
                        output += "\tCritical Threshold: -\n"
                    if "UpperThresholdFatal" in temp:
                        output += "\tFatal Threshold: %s C\n" % temp["UpperThresholdFatal"]
                    else:
                        output += "\tFatal Threshold: -\n"
                    try:
                        if "Health" in temp["Status"]:
                            output += "\tHealth: %s\n" % temp["Status"]["Health"]
                    except KeyError:
                        pass
                    if absent:
                        try:
                            output += "\tState: %s\n" % temp["Status"]["State"]
                        except KeyError:
                            pass
            self.rdmc.ui.printer(output, verbose_override=True)

    def serverinfovalidation(self, options):
        """serverinfo method validation function.

        :param options: command line options
        :type options: list.
        """
        self.cmdbase.login_select_validation(self, options)

    def optionsvalidation(self, options):
        """Checks/updates options.
        :param options: command line options
        :type options: list
        """
        optlist = [
            options.proxy,
            options.firmware,
            options.software,
            options.memory,
            options.thermals,
            options.fans,
            options.power,
            options.processors,
            options.system,
        ]

        if not any(optlist):
            self.setalloptionstrue(options)
        if options.all:
            self.setalloptionstrue(options)
        return options

    def setalloptionstrue(self, options):
        """Updates all selector options values to be True.
        :param options: command line options
        :type options: list
        """
        options.memory = True
        options.thermals = True
        options.firmware = True
        options.software = True
        options.fans = True
        options.processors = True
        options.power = True
        options.system = True
        options.proxy = True
        options.showabsent = True

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        customparser.add_argument(
            "-all",
            "--all",
            dest="all",
            action="store_true",
            help="Add information for all types.",
            default=False,
        )
        customparser.add_argument(
            "-fw",
            "--firmware",
            dest="firmware",
            action="store_true",
            help="Add firmware information to the output.",
            default=False,
        )
        customparser.add_argument(
            "-sw",
            "--software",
            dest="software",
            action="store_true",
            help="Add software information to the output.",
            default=False,
        )
        customparser.add_argument(
            "-memory",
            "--memory",
            dest="memory",
            action="store_true",
            help="Add memory DIMM information to the output.",
            default=False,
        )
        customparser.add_argument(
            "-fans",
            "--fans",
            dest="fans",
            action="store_true",
            help="Add fans information to the output.",
            default=False,
        )
        customparser.add_argument(
            "-processors",
            "--processors",
            dest="processors",
            action="store_true",
            help="Add processor(s) information to the output.",
            default=False,
        )
        customparser.add_argument(
            "-thermals",
            "--thermals",
            dest="thermals",
            action="store_true",
            help="Add thermal information to the output.",
            default=False,
        )
        customparser.add_argument(
            "-power",
            "--power",
            dest="power",
            action="store_true",
            help="Add power information to the output.",
            default=False,
        )
        customparser.add_argument(
            "-system",
            "--system",
            dest="system",
            action="store_true",
            help="Add basic system information to the output.",
            default=False,
        )
        customparser.add_argument(
            "--showabsent",
            dest="showabsent",
            action="store_true",
            help="Include information on absent components in the output.",
            default=False,
        )
        customparser.add_argument(
            "-proxy",
            "--proxy",
            dest="proxy",
            action="store_true",
            help="Add proxy information to the output",
            default=False,
        )
        customparser.add_argument(
            "-j",
            "--json",
            dest="json",
            action="store_true",
            help="Optionally include this flag if you wish to change the"
            " displayed output to JSON format. Preserving the JSON data"
            " structure makes the information easier to parse.",
            default=False,
        )



# ##
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ##

# -*- coding: utf-8 -*-
""" List Component Command for rdmc """

try:
    from rdmc_helper import (
        IncompatibleiLOVersionError,
        InvalidCommandLineErrorOPTS,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        IncompatibleiLOVersionError,
        InvalidCommandLineErrorOPTS,
        ReturnCodes,
    )


class ListComponentCommand:
    """Main download command class"""

    def __init__(self):
        self.ident = {
            "name": "listcomp",
            "usage": None,
            "description": "Run to list the components of " "the currently logged in system.\n\texample: listcomp",
            "summary": "Lists components/binaries from the iLO Repository.",
            "aliases": [],
            "auxcommands": [],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main listcomp worker function

        :param line: string of arguments passed in
        :type line: str.
        :param help_disp: display help flag
        :type line: bool.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, _) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        self.listcomponentvalidation(options)

        if self.rdmc.app.typepath.defs.isgen9:
            raise IncompatibleiLOVersionError("iLO Repository commands are " "only available on iLO 5.")

        comps = self.rdmc.app.getcollectionmembers("/redfish/v1/UpdateService/ComponentRepository/")

        if comps:
            self.printcomponents(comps, options)
        else:
            self.rdmc.ui.printer("No components found.\n")

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def printcomponents(self, comps, options):
        """Print components function

        :param comps: list of components
        :type comps: list.
        """
        if options.json:
            jsonout = dict()
            for comp in comps:
                jsonout[comp["Id"]] = comp
            self.rdmc.ui.print_out_json(jsonout)
        else:
            for comp in comps:
                self.rdmc.ui.printer(
                    "Id: %s\nName: %s\nVersion: %s\nLocked:%s\nComponent "
                    "Uri:%s\nFile Path: %s\nSizeBytes: %s\n\n"
                    % (
                        comp["Id"],
                        comp["Name"],
                        comp["Version"],
                        "Yes" if comp["Locked"] else "No",
                        comp["ComponentUri"],
                        comp["Filepath"],
                        str(comp["SizeBytes"]),
                    )
                )

    def listcomponentvalidation(self, options):
        """listcomp validation function

        :param options: command line options
        :type options: list.
        """
        self.rdmc.login_select_validation(self, options)

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        customparser.add_argument(
            "-j",
            "--json",
            dest="json",
            action="store_true",
            help="Optionally include this flag if you wish to change the"
            " displayed output to JSON format. Preserving the JSON data"
            " structure makes the information easier to parse.",
            default=False,
        )



###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" SendTest Command for rdmc """

try:
    from rdmc_helper import (
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        NoContentsFoundForOperationError,
        ReturnCodes,
    )
except:
    from ilorest.rdmc_helper import (
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        NoContentsFoundForOperationError,
        ReturnCodes,
    )


class SendTestCommand:
    """Send syslog test to the logged in server"""

    def __init__(self):
        self.ident = {
            "name": "sendtest",
            "usage": None,
            "description": "Send syslog test to the "
            "current logged in server.\n\tExample: sendtest syslog\n\n"
            "\tSend alert mail test to the current logged in server.\n\t"
            "sendtest alertmail\n\n\tSend SNMP test alert "
            "to the current logged in server.\n\texample: sendtest snmpalert",
            "summary": "Command for sending various tests to iLO.",
            "aliases": [],
            "auxcommands": [],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main SentTestCommand function

        :param line: string of arguments passed in
        :type line: str.
        :param help_disp: display help flag
        :type line: bool.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        if not len(args) == 1:
            raise InvalidCommandLineError("sendtest command takes only one argument.")

        body = None
        path = None
        actionitem = None

        self.sendtestvalidation(options)

        if args[0].lower() == "snmpalert":
            select = self.rdmc.app.typepath.defs.snmpservice
            actionitem = "SendSNMPTestAlert"
        elif args[0].lower() == "alertmail":
            select = self.rdmc.app.typepath.defs.managernetworkservicetype
            actionitem = "SendTestAlertMail"
        elif args[0].lower() == "syslog":
            select = self.rdmc.app.typepath.defs.managernetworkservicetype
            actionitem = "SendTestSyslog"
        else:
            raise InvalidCommandLineError("sendtest command does not have " "parameter %s." % args[0])

        results = self.rdmc.app.select(selector=select)

        try:
            results = results[0]
        except:
            pass

        if results:
            path = results.resp.request.path
        else:
            raise NoContentsFoundForOperationError("%s not found.It may not " "be available on this system." % select)

        bodydict = results.resp.dict

        try:
            if "Actions" in bodydict:
                for item in bodydict["Actions"]:
                    if actionitem in item:
                        if self.rdmc.app.typepath.defs.isgen10:
                            actionitem = item.split("#")[-1]

                        path = bodydict["Actions"][item]["target"]
                        break
            else:
                for item in bodydict["Oem"][self.rdmc.app.typepath.defs.oemhp]["Actions"]:
                    if actionitem in item:
                        if self.rdmc.app.typepath.defs.isgen10:
                            actionitem = item.split("#")[-1]

                        path = bodydict["Oem"][self.rdmc.app.typepath.defs.oemhp]["Actions"][item]["target"]
                        break

            body = {"Action": actionitem}
        except:
            body = {"Action": actionitem, "Target": "/Oem/Hp"}

        self.rdmc.app.post_handler(path, body)

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def sendtestvalidation(self, options):
        """sendtestvalidation method validation function

        :param options: command line options
        :type options: list.
        """
        self.cmdbase.login_select_validation(self, options)

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)



###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" Factory Reset Controller Command for rdmc """


try:
    from rdmc_helper import (
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        ReturnCodes,
    )


class FactoryResetControllerCommand:
    """Factory reset controller command"""

    def __init__(self):
        self.ident = {
            "name": "factoryresetcontroller",
            "usage": None,
            "description": "Run without "
            "arguments for the current controller options.\n\texample: "
            "factoryresetcontroller\n\n\tTo factory reset a controller "
            "by index.\n\texample: factoryresetcontroller --controller=2"
            '\n\texample: factoryresetcontroller --controller="Slot 1" \n\n'
            "\tTo factory reset a controller on Gen11 server\n"
            "\texample: factoryresetcontroller --reset_type resetall --storageid DE000100\n"
            "\texample: factoryresetcontroller --reset_type preservevolumes --storageid DE000100\n",
            "summary": "Factory resets a controller by index or location.",
            "aliases": [],
            "auxcommands": ["SelectCommand"],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main disk inventory worker function

        :param line: command line input
        :type line: string.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, _) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        self.frcontrollervalidation(options)

        ilo_ver = self.rdmc.app.getiloversion()
        if ilo_ver >= 6.110:
            if not options.storageid:
                raise InvalidCommandLineError(
                    "--storageid option is mandatory for iLO6 along with --controller option.\n"
                )
            if (
                (options.controller is not None)
                and (options.storageid is not None)
                and (options.reset_type is not None)
            ):
                raise InvalidCommandLineError("--controller is not supported in iLO6.")

            elif (options.storageid is not None) and (options.reset_type is not None):
                st_content = []
                self.auxcommands["select"].selectfunction("StorageCollection.")
                get_content = self.rdmc.app.getprops()
                for st_controller in get_content:
                    path = st_controller["Members"]
                    for mem in path:
                        for val in mem.values():
                            if "DE" in val and options.storageid in val:
                                getval = self.rdmc.app.get_handler(val, silent=True, service=True).dict
                                if options.storageid:
                                    if getval["Id"] == options.storageid:
                                        st_content.append(getval)
                                    else:
                                        raise InvalidCommandLineError(
                                            "Selected storage id not found in the current inventory " "list."
                                        )
                                else:
                                    st_content.append(getval)
                for st_controller in st_content:
                    path = st_controller["Actions"]["#Storage.ResetToDefaults"]["target"]

                    if options.reset_type.lower() == "resetall":
                        actionitem = "ResetAll"
                        body = {"ResetType": actionitem}
                    elif options.reset_type.lower() == "preservevolumes":
                        actionitem = "PreserveVolumes"
                        body = {"ResetType": actionitem}
                    else:
                        raise InvalidCommandLineError("Invalid command.")

                    # self.rdmc.ui.printer(
                    #    "FactoryReset path and payload: %s, %s\n"
                    #    % (path, body)
                    # )
                    self.rdmc.app.post_handler(path, body)
                    self.cmdbase.logout_routine(self, options)
                # Return code
                return ReturnCodes.SUCCESS

            elif (options.storageid is None) or (options.reset_type is None):
                raise InvalidCommandLineError("Invalid command\n" "--reset_type and --storageid is required")

        else:
            self.auxcommands["select"].selectfunction("SmartStorageConfig.")
            content = self.rdmc.app.getprops()
            if options.controller and options.reset_type is None and options.storageid is None:
                controllist = []

                try:
                    if options.controller and options.controller.isdigit():
                        slotlocation = self.get_location_from_id(options.controller)
                        if slotlocation:
                            slotcontrol = slotlocation.lower().strip('"').split("slot")[-1].lstrip()
                            for control in content:
                                if slotcontrol.lower() == control["Location"].lower().split("slot")[-1].lstrip():
                                    controllist.append(control)
                    # else:
                    #    self.parser.print_help()
                    #    return ReturnCodes.SUCCESS
                    if not controllist:
                        raise InvalidCommandLineError("")
                except InvalidCommandLineError:
                    raise InvalidCommandLineError("Selected controller not found in the current inventory " "list.")
                for controller in controllist:
                    contentsholder = {
                        "Actions": [{"Action": "FactoryReset"}],
                        "DataGuard": "Disabled",
                    }
                    self.rdmc.ui.printer(
                        "FactoryReset path and payload: %s, %s\n" % (controller["@odata.id"], contentsholder)
                    )
                    self.rdmc.app.patch_handler(controller["@odata.id"], contentsholder)

            elif options.controller and (options.reset_type is not None) or (options.storageid is not None):
                raise InvalidCommandLineError(
                    "Invalid command\n" "--reset_type and --storageid is not supported in iLO5"
                )

            for idx, val in enumerate(content):
                self.rdmc.ui.printer("[%d]: %s\n" % (idx, val["Location"]))

            self.cmdbase.logout_routine(self, options)
            # Return code
            return ReturnCodes.SUCCESS

    def get_location_from_id(self, controller_id):
        for sel in self.rdmc.app.select("SmartStorageArrayController", path_refresh=True):
            if "Collection" not in sel.maj_type:
                controller = sel.dict
                if controller["Id"] == str(controller_id):
                    return controller["Location"]
        return None

    def frcontrollervalidation(self, options):
        """Factory reset controller validation function

        :param options: command line options
        :type options: list.
        """
        self.cmdbase.login_select_validation(self, options)

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """

        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        customparser.add_argument(
            "--controller",
            dest="controller",
            help="Use this flag to select the corresponding controller " "using either the slot number or index.",
            default=None,
        )

        customparser.add_argument(
            "--reset_type",
            dest="reset_type",
            help="Use this flag to pass the reset type for storage controller  in iLO6 ",
            default=None,
        )

        customparser.add_argument(
            "--storageid",
            dest="storageid",
            help="Use this flag to select the corresponding storage id in iLO6",
            default=None,
        )



# ##
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ##

# -*- coding: utf-8 -*-
""" Install Set Command for rdmc """

import json
import re
from argparse import RawDescriptionHelpFormatter
from datetime import datetime

try:
    from rdmc_helper import (
        LOGGER,
        IncompatibleiLOVersionError,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        InvalidFileInputError,
        NoContentsFoundForOperationError,
        ReturnCodes,
        InstallsetError,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        LOGGER,
        IncompatibleiLOVersionError,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        InvalidFileInputError,
        NoContentsFoundForOperationError,
        ReturnCodes,
        InstallsetError,
    )


class InstallSetCommand:
    """Main download command class"""

    def __init__(self):
        self.ident = {
            "name": "installset",
            "usage": None,
            "description": "Run to perform operations on install sets.\nTo view help on specific "
            "sub-commands run: installset <sub-command> -h\n\n\tExample: installset "
            "add -h\nNote: iLO 5 required.",
            "summary": "Manages install sets for iLO.",
            "aliases": [],
            "auxcommands": [],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main listcomp worker function

        :param line: string of arguments passed in
        :type line: str.
        :param help_disp: display help flag
        :type line: bool.
        """
        if help_disp:
            line.append("-h")
            try:
                (_, _) = self.rdmc.rdmc_parse_arglist(self, line)
            except:
                return ReturnCodes.SUCCESS
            return ReturnCodes.SUCCESS
        try:
            _subcommands = ["add", "delete", "invoke"]
            found = False
            for i, arg in enumerate(line):
                if arg in _subcommands:
                    found = True
                    try:
                        if line[i + 1] not in list(self.parser._option_string_actions.keys()):
                            (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
                        else:
                            raise IndexError
                    except (KeyError, IndexError):
                        (options, args) = self.rdmc.rdmc_parse_arglist(self, line, default=True)
                    else:
                        continue
            if not found:
                (options, args) = self.rdmc.rdmc_parse_arglist(self, line, default=True)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        self.installsetvalidation(options)

        if self.rdmc.app.typepath.defs.isgen9:
            raise IncompatibleiLOVersionError("iLO Repository commands are " "only available on iLO 5.")

        if hasattr(options, "name"):
            if options.name:
                if options.name.startswith('"') and options.name.endswith('"'):
                    options.name = options.name[1:-1]
        if hasattr(options, "removeall"):
            if options.removeall:
                self.removeinstallsets(options)
        if options.command.lower() == "add":
            if not options.json:
                raise InvalidCommandLineError("add command requires an install set file.")
            else:
                self.addinstallset(options.json, options)
        elif options.command.lower() == "delete" and (hasattr(options, "removeall") and not options.removeall):
            self.deleteinstallset(options)
        # elif options.command.lower() == "delete" and not hasattr(options, "removeall"):
        #     self.deleteinstallset(options.name)
        elif options.command.lower() == "invoke":
            self.invokeinstallset(options)
        else:
            self.printinstallsets(options)

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def addinstallset(self, setfile, options):
        """Adds an install set
        :param setfile: filename of install set
        :type setfile: str.
        :param name: string of the install set name to add
        :type name: str.
        """
        path = "/redfish/v1/UpdateService/InstallSets/"
        comps = self.rdmc.app.getcollectionmembers("/redfish/v1/UpdateService/ComponentRepository/")

        sets = self.rdmc.app.getcollectionmembers(path)

        if not options.name:
            options.name = str(datetime.now())
            options.name = options.name.replace(":", "")
            options.name = options.name.replace(" ", "T")
        try:
            inputfile = open(setfile, "r")
            sequences = json.loads(inputfile.read())
        except Exception as excp:
            raise InvalidFileInputError("%s" % excp)

        listtype = self.validatefile(sequences)

        if listtype:
            body = {"Name": options.name, "Sequence": sequences}
        else:
            body = sequences
            if "Sequence" in body:
                sequences = body["Sequence"]
            else:
                raise InvalidFileInputError("Invalid install set file.")
            sequences = body["Sequence"]

        filenamelist = [x["Filename"] for x in comps]

        for sequence in sequences:
            if sequence["Command"] == "ApplyUpdate":
                if "Filename" in list(sequence.keys()):
                    if not sequence["Filename"] in filenamelist:
                        raise NoContentsFoundForOperationError(
                            "Component"
                            " referenced in install set is not present on"
                            " iLO Drive: %s" % sequence["Filename"]
                        )
            elif sequence["Command"] == "WaitTimeSeconds":
                sequence["WaitTimeSeconds"] = int(sequence["WaitTimeSeconds"])
        for setvar in sets:
            if setvar["Name"] == body["Name"]:
                raise InvalidCommandLineError("Install set name is already in use.")

        # self.rdmc.app.post_handler(path, body)
        if "IsRecovery" in body and body["IsRecovery"]:
            self.rdmc.ui.printer("This is a system recovery install set: %s\n" % body["Name"])
            self.rdmc.ui.warn("Adding system recovery set: %s\n" % body["Name"])
            if not (options.user and options.password):
                raise InstallsetError(
                    "Adding system recovery set needs to be passed with " "--user and --password options, add failed\n"
                )
            if "blobstore" in self.rdmc.app.current_client.base_url:
                LOGGER.info("Logging out of the session without user and password")
                self.rdmc.app.current_client.logout()
                LOGGER.info("Logging in with user and password for deleting system recovery set")
                self.rdmc.app.current_client._user_pass = (options.user, options.password)
                self.rdmc.app.current_client.login(self.rdmc.app.current_client.auth_type)
                self.rdmc.app.post_handler(path, body)
            else:
                self.rdmc.app.post_handler(path, body)
        else:
            self.rdmc.app.post_handler(path, body)

    def invokeinstallset(self, options):
        """Invokes the named set
        :param name: string of the install set name to invoke
        :type name: str.
        """
        path = None
        name = options.name
        sets = self.rdmc.app.getcollectionmembers("/redfish/v1/UpdateService/InstallSets/")

        for setvar in sets:
            if setvar["Name"] == name:
                path = setvar["Actions"]["#HpeComponentInstallSet.Invoke"]["target"]

        if not path:
            raise NoContentsFoundForOperationError("No install set with the" " provided name could be found.")

        self.rdmc.ui.printer("Invoking install set:%s\n" % name)

        payload = self.checkpayloadoptions(options)
        self.rdmc.app.post_handler(path, payload)

    def deleteinstallset(self, options):
        """Deletes the named set

        :param name: string of the install set name to delete
        :type name: str.
        """
        self.rdmc.ui.printer("Deleting install set: %s...\n" % options.name)
        sets = self.rdmc.app.getcollectionmembers("/redfish/v1/UpdateService/InstallSets/")
        found = False

        for setvar in sets:
            if setvar["Name"] == options.name:
                found = True
                path = setvar["@odata.id"]
                if "IsRecovery" in setvar and setvar["IsRecovery"]:
                    self.rdmc.ui.printer("This is a system recovery install set: %s\n" % setvar["Name"])
                    self.rdmc.ui.warn("Deleting system recovery set: %s\n" % setvar["Name"])
                    if not (options.user and options.password):
                        raise InstallsetError(
                            "Deleting system recovery set needs to be passed with "
                            "--user and --password options, delete failed\n"
                        )
                    if "blobstore" in self.rdmc.app.current_client.base_url:
                        LOGGER.info("Logging out of the session without user and password")
                        self.rdmc.app.current_client.logout()
                        LOGGER.info("Logging in with user and password for deleting system recovery set")
                        self.rdmc.app.current_client._user_pass = (options.user, options.password)
                        self.rdmc.app.current_client.login(self.rdmc.app.current_client.auth_type)
                        self.rdmc.app.delete_handler(path)
                    else:
                        self.rdmc.app.delete_handler(path)
                else:
                    self.rdmc.app.delete_handler(path)
        if not found:
            raise NoContentsFoundForOperationError("Unable to find the specified install set.")

    def removeinstallsets(self, options):
        """Removes all install sets"""
        sets = self.rdmc.app.getcollectionmembers("/redfish/v1/UpdateService/InstallSets/")

        if not sets:
            self.rdmc.ui.printer("No install sets found.\n")

        self.rdmc.ui.printer("Deleting all install sets...\n")

        for setvar in sets:
            if setvar["IsRecovery"]:
                self.rdmc.ui.warn("Attempting to delete recovery set: %s\n" % setvar["Name"])
                if not (options.user and options.password):
                    raise InstallsetError(
                        "Deleting system recovery set needs to be passed with "
                        "--user and --password options, delete failed\n"
                    )
                self.rdmc.ui.warn("Deleting system recovery set: %s\n" % setvar["Name"])
                LOGGER.debug(
                    "This is to inform that ,Recovery install set %s will also be "
                    "deleted as RemoveAll option is being used\n" % setvar["Name"]
                )

                if "blobstore" in self.rdmc.app.current_client.base_url:
                    LOGGER.info("Logging out of the session without user and password")
                    self.rdmc.app.current_client.logout()
                    LOGGER.info("Logging in with user and password for deleting system recovery set")
                    self.rdmc.app.current_client._user_pass = (options.user, options.password)
                    self.rdmc.app.current_client.login(self.rdmc.app.current_client.auth_type)
                    self.rdmc.app.delete_handler(setvar["@odata.id"])
                else:
                    self.rdmc.app.delete_handler(setvar["@odata.id"])
            else:
                self.rdmc.ui.printer("Deleting install set: %s\n" % setvar["Name"])
                self.rdmc.app.delete_handler(setvar["@odata.id"])

    def build_json_out(self, options):
        sets = self.rdmc.app.getcollectionmembers("/redfish/v1/UpdateService/InstallSets/")
        list_content = dict()
        for setvar in sets:
            content = dict()
            content[setvar["Name"]] = dict()
            content[setvar["Name"]].update({"IsRecovery": setvar["IsRecovery"]})
            list_content.update(content)

            if "Sequence" not in list(setvar.keys()):
                content.update({"No Sequences in set."})
                continue

            for item in setvar["Sequence"]:
                if "Filename" in list(item.keys()):
                    name = item["Name"]
                    filename_cmd = "%s %s" % (item["Command"], item["Filename"])
                    content[setvar["Name"]].update({name: {filename_cmd}})
                elif "WaitTimeSeconds" in list(item.keys()):
                    wait_name = item["Name"]
                    wait_command = "%s %s seconds" % (
                        item["Command"],
                        str(item["WaitTimeSeconds"]),
                    )
                    content[setvar["Name"]].update({wait_name: {wait_command}})
                else:
                    content[setvar["Name"]].update({item["Name"]: {item["Command"]}})
        return list_content

    def printinstallsets(self, options):
        """Prints install sets"""
        sets = self.rdmc.app.getcollectionmembers("/redfish/v1/UpdateService/InstallSets/")
        if not options.json:
            self.rdmc.ui.printer("Install Sets:\n")

        if not sets:
            self.rdmc.ui.printer("No install sets found.\n")
        elif not options.json:
            for setvar in sets:
                if setvar["IsRecovery"]:
                    recovery = "[Recovery Set]"
                else:
                    recovery = ""
                self.rdmc.ui.printer("\n%s: %s\n" % (setvar["Name"], recovery))

                if "Sequence" not in list(setvar.keys()):
                    self.rdmc.ui.warn("\tNo Sequences in set.\n")
                    continue

                for item in setvar["Sequence"]:
                    if "Filename" in list(item.keys()):
                        self.rdmc.ui.printer("\t%s: %s %s\n" % (item["Name"], item["Command"], item["Filename"]))
                    elif "WaitTimeSeconds" in list(item.keys()):
                        self.rdmc.ui.printer(
                            "\t%s: %s %s seconds\n"
                            % (
                                item["Name"],
                                item["Command"],
                                str(item["WaitTimeSeconds"]),
                            )
                        )
                    else:
                        self.rdmc.ui.printer("\t%s: %s\n" % (item["Name"], item["Command"]))
        elif options.json:
            json_content = self.build_json_out(options)
            self.rdmc.ui.print_out_json(json_content)

    def validatefile(self, installsetfile):
        """validates json file

        :param file: json file to validate
        :type file: string.
        """
        listtype = True
        keylist = ["Name", "UpdatableBy", "Command", "WaitTimeSeconds", "Filename"]

        if isinstance(installsetfile, list):
            for item in installsetfile:
                for key in item:
                    if key not in keylist:
                        raise InvalidFileInputError("Property %s is invalid " "for an install set." % key)
        else:
            listtype = False

        return listtype

    def checkpayloadoptions(self, options):
        """checks for payload options and adds needed ones to payload

        :param options: command line options
        :type options: list.
        """

        payload = {}
        if options.exafter:
            self.checktime(options.exafter)
            payload["Expire"] = options.exafter
        if options.safter:
            self.checktime(options.safter)
            payload["StartAfter"] = options.safter
        if options.tover:
            payload["TPMOverride"] = True
        if options.uset:
            payload["UpdateRecoverySet"] = True
        if options.ctakeq:
            payload["ClearTaskQueue"] = True

        return payload

    def checktime(self, timestr):
        """check the time string for valid format

        :param timestr: time string to check
        :type timestr: string.
        """
        rfdtregex = "\\d\\d\\d\\d-\\d\\d-\\d\\dT\\d\\d:\\d\\d:\\d\\dZ?"
        if not re.match(rfdtregex, timestr):
            raise InvalidCommandLineError(
                "Invalid redfish date-time format. " "Accepted formats: YYYY-MM-DDThh:mm:ss, YYYY-MM-DDThh:mm:ssZ"
            )

    def installsetvalidation(self, options):
        """installset validation function

        :param options: command line options
        :type options: list.
        """
        self.cmdbase.login_select_validation(self, options)

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)
        subcommand_parser = customparser.add_subparsers(dest="command")
        # default sub-parser
        default_parser = subcommand_parser.add_parser(
            "default",
            help="Running without any sub-command will return a list of all available install "
            "sets on the currently logged in server.",
        )
        default_parser.add_argument(
            "-j",
            "--json",
            dest="json",
            action="store_true",
            help="Optionally include this flag if you wish to change the"
            " displayed output to JSON format. Preserving the JSON data"
            " structure makes the information easier to parse.",
            default=False,
        )
        default_parser.add_argument(
            "--removeall",
            dest="removeall",
            help="Optionally include this flag to remove all install sets on the currently logged " "in server.",
            action="store_true",
            default=False,
        )
        self.cmdbase.add_login_arguments_group(default_parser)

        # add sub-parser
        add_help = "Adds an install set on the currently logged in server."
        add_parser = subcommand_parser.add_parser(
            "add",
            help=add_help,
            description=add_help + "\n\texample: installset add installsetfile.json "
            "--name=newinstallsetname.\n\n\tNote: iLO will provide a default "
            "install set name if the flag '--name' is not provided.",
            formatter_class=RawDescriptionHelpFormatter,
        )
        add_parser.add_argument(
            "json",
            help="Json file containing the install set tasks and sequencing"
            '\n\n\texample install set JSON file:\n\t[\n\t\t{\n\t\t\t"Name": '
            '"Wait",\n\t\t\t"UpdatableBy": ["RuntimeAgent"],\n\t\t\t'
            '"Command": "Wait",\n\t\t\t"WaitTimeSeconds": 60\n\t\t},\n\t\t{'
            '\n\t\t\t"Name": "uniqueName",\n\t\t\t"UpdatableBy": '
            '["RuntimeAgent"],\n\t\t\t"Command": "ApplyUpdate",\n\t\t\t"'
            'Filename": "filename.exe"\n\t\t},\n\t\t{\n\t\t\t"Name": '
            '"Reboot",\n\t\t\t"UpdatableBy": ["RuntimeAgent"],\n\t\t\t'
            '"Command": "ResetServer"\n\t\t}\n\t]',
            metavar="JSONFILE",
        )
        add_parser.add_argument("-n", "--name", dest="name", help="Install set name.", default=None)

        self.cmdbase.add_login_arguments_group(add_parser)

        # invoke sub-parser
        invoke_help = "Invoke execution of an install script on the currently logged in server."
        invoke_parser = subcommand_parser.add_parser(
            "invoke",
            help=invoke_help,
            description=invoke_help + "\n\nExamples:\n\tTo simply invoke an install set\n\t"
            "installset invoke --name=installsetname",
            formatter_class=RawDescriptionHelpFormatter,
        )
        invoke_parser.add_argument(
            "-n",
            "--name",
            help="Install set name.",
            metavar="NAME",
            nargs="?",
            default="",
            required=True,
        )
        invoke_parser.add_argument(
            "--cleartaskqueue",
            dest="ctakeq",
            action="store_true",
            help="This option allows previous items in the task queue to"
            " be cleared before the Install Set is invoked",
            default=False,
        )
        invoke_parser.add_argument(
            "--expire",
            dest="exafter",
            help="Optionally include this flag if you wish to set the"
            " time for installset to expire. ISO 8601 Redfish-style time "
            "string to be written after which iLO will automatically change "
            "state to Expired",
            default=None,
        )
        invoke_parser.add_argument(
            "--startafter",
            dest="safter",
            help="Optionally include this flag if you wish to set the"
            " earliest execution time for installset. ISO 8601 Redfish-style "
            "time string to be used.",
            default=None,
        )
        invoke_parser.add_argument(
            "--tpmover",
            dest="tover",
            action="store_true",
            help="If set then the TPMOverrideFlag is passed in on the " "associated flash operations",
            default=False,
        )
        invoke_parser.add_argument(
            "--updaterecoveryset",
            dest="uset",
            action="store_true",
            help="If set then the components in the flash operations are used"
            " to replace matching contents in the Recovery Set.",
            default=False,
        )
        self.cmdbase.add_login_arguments_group(invoke_parser)

        # delete sub-parser
        delete_help = "Delete one or more install sets from the currently logged in server."
        delete_parser = subcommand_parser.add_parser(
            "delete",
            help=delete_help,
            description=delete_help + "\n\nExamples:\n\nTo delete a single install set:\n\t"
            "installset delete --name=installsetname.\n\nTo delete all install sets\n\t"
            "installset delete --removeall",
            formatter_class=RawDescriptionHelpFormatter,
        )
        delete_parser.add_argument(
            "-n",
            "--name",
            help="Install set name",
            metavar="NAME",
            nargs="?",
            default="",
        )
        delete_parser.add_argument(
            "--removeall",
            dest="removeall",
            help="Optionally include this flag to remove all install sets on the currently logged " "in server.",
            action="store_true",
            default=False,
        )
        self.cmdbase.add_login_arguments_group(delete_parser)



# ##
# Copyright 2016 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# BMN
# ##

# -*- coding: utf-8 -*-
""" Get Inventory Command for rdmc """


try:
    from rdmc_helper import (
        IncompatibleiLOVersionError,
        InvalidCommandLineErrorOPTS,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        IncompatibleiLOVersionError,
        InvalidCommandLineErrorOPTS,
        ReturnCodes,
    )

from redfish.ris.resp_handler import ResponseHandler


class GetInventoryCommand:
    """GetInventory command class"""

    def __init__(self):
        self.ident = {
            "name": "getinventory",
            "usage": None,
            "description": "Get complete inventory"
            "data from the iLO, including iLO Repo and install set."
            "\n\texample: getinventory all",
            "summary": "Get complete inventory data from the iLO.",
            "aliases": [],
            "auxcommands": [],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()
        self.response_handler = None

    def run(self, line, help_disp=False):
        """Main GetInventory worker function
        :param line: string of arguments passed in
        :type line: str.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, _) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        self.getinventoryvalidation(options)
        self.response_handler = ResponseHandler(
            self.rdmc.app.validationmanager,
            self.rdmc.app.typepath.defs.messageregistrytype,
        )
        if self.rdmc.app.typepath.defs.isgen9:
            raise IncompatibleiLOVersionError("GetInventory command is " "available only on iLO 5 node.")

        alldata = {}
        results = {}

        # If both are not set, then we will get both inventory and repo data
        if not options.inventory and not options.repo_data and not options.sut:
            options.inventory = True
            options.repo_data = True
            options.sut = True

        if options.inventory:
            try:
                collectiondata = self.rdmc.app.get_handler(
                    "/redfish/v1/updateservice/firmwareinventory", service=False, silent=True
                ).dict
                members = self.rdmc.app.getcollectionmembers(collectiondata.get("@odata.id"))
                collectiondata.update({"Members": members})
                alldata.update({"firmwareInventory": collectiondata})
            except:
                alldata.update({"firmwareInventory": {}})
            try:
                collectiondata = self.rdmc.app.get_handler(
                    "/redfish/v1/updateservice/softwareinventory", service=False, silent=True
                ).dict
                members = self.rdmc.app.getcollectionmembers(collectiondata.get("@odata.id"))
                collectiondata.update({"Members": members})
                alldata.update({"softwareInventory": collectiondata})
            except:
                alldata.update({"softwareInventory": {}})

            try:
                members = self.rdmc.app.get_handler("/redfish/v1/systems/", service=False, silent=True).dict["Members"]
                systemdata = {}
                rootsystem = {}
                for id in members:
                    for mem_id in id.values():
                        results = self.rdmc.app.get_handler(mem_id, service=True, silent=True).dict
                        id = results["Id"]
                        if id != "1":
                            rootsystem.update({"systems%s" % id: results})
                            systemdata.update({"systems": rootsystem})
                            alldata.update(systemdata)
                        if id == "1":
                            alldata.update({"systems1": results})
            except:
                alldata.update({"systems": {}})

            try:
                sutlocation = alldata.get("systems1").get("Oem").get("Hpe").get("Links")
            except:
                if results:
                    sutlocation = next(iter(results)).dict.get("Links")
            for key, value in list(sutlocation.items()):
                if key == "SUT":
                    result2 = self.rdmc.app.get_handler(value.get("@odata.id"), service=False, silent=True)

                    if result2.status == 200:
                        alldata["SUT"] = result2.dict
                    else:
                        return self.printlastfailedresult(result2)
                    break
            try:
                collectiondata = self.rdmc.app.get_handler(
                    "/redfish/v1/Managers/1/EthernetInterfaces/", service=False, silent=True
                ).dict
                members = self.rdmc.app.getcollectionmembers(collectiondata.get("@odata.id"))
                collectiondata.update({"Members": members})
                alldata.update({"EthernetInterfaces": collectiondata})
            except:
                alldata.update({"EthernetInterfaces": {}})

            try:
                results = self.rdmc.app.select(selector="HpeiLODateTime.", path_refresh=True)
                alldata["DateTime"] = next(iter(results)).resp.dict
            except:
                alldata.update({"DateTime": {}})

            try:
                collectiondata = self.rdmc.app.get_handler(
                    "/redfish/v1/UpdateService/", service=False, silent=True
                ).dict
                alldata.update({"DowngradePolicy": collectiondata})
            except:
                alldata.update({"DowngradePolicy": {}})

        if options.sut:
            if "system1" not in alldata:
                try:
                    results = self.rdmc.app.get_handler("/redfish/v1/systems/", service=False, silent=True).dict
                    members = results["Members"]
                    for id in members:
                        for mem_id in id.values():
                            results = self.rdmc.app.get_handler(mem_id, service=True, silent=True).dict
                            id = results["Id"]
                            if id == "1":
                                alldata.update({"systems1": results})
                except:
                    alldata.update({"systems1": {}})
            else:
                _ = alldata["systems1"]

            if alldata.get("systems1"):
                try:
                    sutlocation = alldata.get("systems1").get("Oem").get("Hpe").get("Links")
                except:
                    if results:
                        sutlocation = next(iter(results)).dict.get("Links")
                for key, value in list(sutlocation.items()):
                    if key == "SUT":
                        result2 = self.rdmc.app.get_handler(value.get("@odata.id"), service=False, silent=True)

                        if result2.status == 200:
                            alldata["SUT"] = result2.dict
                        else:
                            return self.printlastfailedresult(result2)
                        break

        if options.repo_data:
            try:
                collectiondata = self.rdmc.app.get_handler(
                    "/redfish/v1/UpdateService/installsets", service=False, silent=True
                ).dict
                members = self.rdmc.app.getcollectionmembers(collectiondata.get("@odata.id"))
                if len(members) == 0:
                    members = []
                collectiondata.update({"Members": members})
                alldata.update({"installsets": collectiondata})
            except:
                alldata.update({"installsets": []})

            try:
                collectiondata = self.rdmc.app.get_handler(
                    "/redfish/v1/UpdateService/updatetaskqueue", service=False, silent=True
                ).dict
                members = self.rdmc.app.getcollectionmembers(collectiondata.get("@odata.id"))
                if len(members) == 0:
                    members = []
                collectiondata.update({"Members": members})
                alldata.update({"updatetaskqueue": collectiondata})
            except:
                alldata.update({"updatetaskqueue": []})

            if self.rdmc.app.getiloversion(skipschemas=True) >= 5.130:
                try:
                    collectiondata = self.rdmc.app.get_handler(
                        "/redfish/v1/UpdateService/maintenancewindows", service=False, silent=True
                    ).dict
                    members = self.rdmc.app.getcollectionmembers(collectiondata.get("@odata.id"))
                    if len(members) == 0:
                        members = []
                    collectiondata.update({"Members": members})
                    alldata.update({"maintenancewindows": collectiondata})
                except:
                    alldata.update({"maintenancewindows": []})

            comp_repo_url = "/redfish/v1/UpdateService/ComponentRepository/" + "?$expand=."
            try:
                members = self.rdmc.app.get_handler(comp_repo_url, silent=True, service=True).dict
                alldata.update({"ComponentRepository": members})
            except:
                alldata.update({"ComponentRepository": {}})

        self.rdmc.ui.print_out_json(alldata)

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def combineinstances(self, data, alldata, name, coll=None):
        """combine the data into one json dictionary"""
        if coll:
            coll = coll.resp.dict
            finaldata = []
            if "Members" in coll:
                for member in data:
                    finaldata.append(member.resp.dict)
                coll["Members"] = finaldata
            alldata[name] = coll
        elif len(data) > 1:
            alldata.update({name: {"Members": []}})
            for item in data:
                alldata[name]["Members"].append(item.resp.dict)
        else:
            alldata.update({name: {"Members": next(iter(data)).resp.dict}})

    def printlastfailedresult(self, results):
        """print last failed result function"""
        self.response_handler.output_resp(results)

        return ReturnCodes.NO_CONTENTS_FOUND_FOR_OPERATION

    def getinventoryvalidation(self, options):
        """get inventory validation function
        :param options: command line options
        :type options: list.
        """
        self.cmdbase.login_select_validation(self, options)

    def definearguments(self, customparser):
        """Wrapper function for new command main function
        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        customparser.add_argument(
            "-i",
            "--inventory",
            dest="inventory",
            action="store_true",
            help="""Use this option to get only the inventory data.""",
            default=False,
        )
        customparser.add_argument(
            "-s",
            "--sut",
            dest="sut",
            action="store_true",
            help="""Use this option to get only SUT data from iLO.""",
            default=False,
        )
        customparser.add_argument(
            "-r",
            "--repo_data",
            dest="repo_data",
            action="store_true",
            help="""Use this option to get only iLO repository, install set """ """and Task queue details.""",
            default=False,
        )



###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" Add Account Command for rdmc """

import getpass
import os
from argparse import Action, RawDescriptionHelpFormatter

from redfish.ris.ris import SessionExpired
from redfish.ris.rmc_helper import IdTokenError

try:
    from rdmc_helper import (
        Encryption,
        IncompatibleiLOVersionError,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        InvalidFileInputError,
        NoContentsFoundForOperationError,
        ResourceExists,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        Encryption,
        IncompatibleiLOVersionError,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        InvalidFileInputError,
        NoContentsFoundForOperationError,
        ResourceExists,
        ReturnCodes,
    )

__subparsers__ = ["add", "modify", "changepass", "delete", "addcert", "deletecert"]


class _AccountParse(Action):
    def __init__(self, option_strings, dest, nargs, **kwargs):
        super(_AccountParse, self).__init__(option_strings, dest, nargs, **kwargs)

    def __call__(self, parser, namespace, values, option_strings):
        """Account privileges option helper"""

        privkey = {
            1: "LoginPriv",
            2: "RemoteConsolePriv",
            3: "UserConfigPriv",
            4: "iLOConfigPriv",
            5: "VirtualMediaPriv",
            6: "VirtualPowerAndResetPriv",
            7: "HostNICConfigPriv",
            8: "HostBIOSConfigPriv",
            9: "HostStorageConfigPriv",
            10: "SystemRecoveryConfigPriv",
        }

        for priv in next(iter(values)).split(","):
            try:
                priv = int(priv)
            except ValueError:
                try:
                    parser.error("Invalid privilege entered: %s. Privileges must " "be numbers." % priv)
                except:
                    raise InvalidCommandLineErrorOPTS("")
            try:
                if not isinstance(namespace.optprivs, list):
                    namespace.optprivs = list()
                if option_strings.startswith("--add"):
                    namespace.optprivs.append({privkey[priv]: True})
                elif option_strings.startswith("--remove"):
                    namespace.optprivs.append({privkey[priv]: False})
            except KeyError:
                try:
                    parser.error(
                        "Invalid privilege entered: %s. Number does not " "match an available privilege." % priv
                    )
                except:
                    raise InvalidCommandLineErrorOPTS("")

    # account_parse.counter = 0


class IloAccountsCommand:
    """command to manipulate/add ilo user accounts"""

    def __init__(self):
        self.ident = {
            "name": "iloaccounts",
            "usage": None,
            "description": "\tView, Add, Remove, and Modify iLO accounts based on the "
            "sub-command used.\n\n\tTo view help on specific sub-commands run: "
            "iloaccounts <sub-command> -h\n\t"
            "Example: iloaccounts add -h\n\tNote: \n\t\t1. UserName and LoginName are reversed "
            "in the iLO GUI for Redfish compatibility.\n\t\t"
            "2. While executing the command: iloaccounts add in a Linux machine, "
            "an escape character needs to be added before a special character in the password.\n\t\t\t"
            "Example: iloaccount add rest rest 12iso*help\n",
            "summary": "Views/Adds/deletes/modifies an iLO account on the currently logged in server.",
            "aliases": ["iloaccount"],
            "auxcommands": [],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main iloaccounts function

        :param line: string of arguments passed in
        :type line: str.
        :param help_disp: display help flag
        :type line: bool.
        """
        if help_disp:
            line.append("-h")
            try:
                (_, _) = self.rdmc.rdmc_parse_arglist(self, line)
            except:
                return ReturnCodes.SUCCESS
            return ReturnCodes.SUCCESS
        acct = mod_acct = None
        try:
            ident_subparser = False
            for cmnd in __subparsers__:
                if cmnd in line:
                    (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
                    ident_subparser = True
                    break
            if not ident_subparser:
                (options, args) = self.rdmc.rdmc_parse_arglist(self, line, default=True)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        self.iloaccountsvalidation(options)

        redfish = self.rdmc.app.monolith.is_redfish
        path = self.rdmc.app.typepath.defs.accountspath
        results = self.rdmc.app.get_handler(path, service=True, silent=True).dict

        if redfish:
            results = results["Members"]
        else:
            if "Member" in results["links"]:
                results = results["links"]["Member"]
            else:
                results = list()

        for indx, acct in enumerate(results):
            acct = self.rdmc.app.get_handler(
                acct[self.rdmc.app.typepath.defs.hrefstring], service=True, silent=True
            ).dict
            try:
                if hasattr(options, "identifier"):
                    if acct["Id"] == options.identifier or acct["UserName"] == options.identifier:
                        if redfish:
                            path = acct["@odata.id"]
                        else:
                            path = acct["links"]["self"]["href"]
                        mod_acct = acct
                elif options.command == "default":
                    results[indx] = acct
                else:
                    raise KeyError
            except KeyError:
                continue
            else:
                if mod_acct:
                    acct = mod_acct
                    break
                else:
                    acct = None

        if not results:
            raise NoContentsFoundForOperationError("No iLO Management Accounts were found.")

        outdict = dict()
        if options.command.lower() == "default":
            if not options.json:
                self.rdmc.ui.printer(
                    "\niLO Account info:\n\n[Id] UserName (LoginName): " "\nPrivileges\n-----------------\n\n",
                    verbose_override=True,
                )
            for acct in sorted(results, key=lambda k: int(k["Id"])):
                privstr = ""
                privs = acct["Oem"][self.rdmc.app.typepath.defs.oemhp]["Privileges"]

                if (
                    "ServiceAccount" in list(acct["Oem"][self.rdmc.app.typepath.defs.oemhp].keys())
                    and acct["Oem"][self.rdmc.app.typepath.defs.oemhp]["ServiceAccount"]
                ):
                    service = "ServiceAccount=True"
                else:
                    service = "ServiceAccount=False"
                if not options.json:
                    for priv in privs:
                        privstr += priv + "=" + str(privs[priv]) + "\n"
                    self.rdmc.ui.printer(
                        "[%s] %s (%s):\n%s\n%s\n"
                        % (
                            acct["Id"],
                            acct["UserName"],
                            acct["Oem"][self.rdmc.app.typepath.defs.oemhp]["LoginName"],
                            service,
                            privstr,
                        ),
                        verbose_override=True,
                    )
                keyval = "[" + str(acct["Id"]) + "] " + acct["UserName"]
                outdict[keyval] = privs
                outdict[keyval]["ServiceAccount"] = service.split("=")[-1].lower()
            if options.json:
                self.rdmc.ui.print_out_json_ordered(outdict)
        elif options.command.lower() == "changepass":
            if not acct:
                raise InvalidCommandLineError("Unable to find the specified account.")
            if not options.acct_password:
                self.rdmc.ui.printer("Please input the new password.\n", verbose_override=True)
                tempinput = getpass.getpass()
                self.credentialsvalidation("", "", tempinput, "", True, options)
                options.acct_password = tempinput

            self.credentialsvalidation("", "", options.acct_password.split("\r")[0], "", True)
            body = {"Password": options.acct_password.split("\r")[0]}

            if path and body:
                self.rdmc.app.patch_handler(path, body)
            else:
                raise NoContentsFoundForOperationError("Unable to find the specified account.")

        elif options.command.lower() == "add":
            if options.encode:
                args[2] = Encryption.decode_credentials(args[2])
                if isinstance(args[2], bytes):
                    args[2] = args[2].decode("utf-8")

            privs = self.getprivs(options)
            path = self.rdmc.app.typepath.defs.accountspath

            body = {
                "UserName": options.identifier,
                "Password": options.acct_password,
                "Oem": {self.rdmc.app.typepath.defs.oemhp: {"LoginName": options.loginname}},
            }
            if privs:
                body["Oem"][self.rdmc.app.typepath.defs.oemhp].update({"Privileges": privs})
            self.credentialsvalidation(options.identifier, options.loginname, options.acct_password, acct, True)
            if options.serviceacc:
                body["Oem"][self.rdmc.app.typepath.defs.oemhp].update({"ServiceAccount": True})
            if options.role:
                if self.rdmc.app.getiloversion() >= 5.140:
                    body["RoleId"] = options.role
                else:
                    raise IncompatibleiLOVersionError("Roles can only be set in iLO 5" " 1.40 or greater.")
            if path and body:
                self.rdmc.app.post_handler(path, body)
            self.rdmc.ui.printer("New iLO account %s is added successfully\n" % options.identifier)
        elif options.command.lower() == "modify":
            if not mod_acct:
                raise InvalidCommandLineError("Unable to find the specified account.")
            body = {}

            if options.optprivs:
                body.update({"Oem": {self.rdmc.app.typepath.defs.oemhp: {"Privileges": {}}}})
                if any(
                    priv for priv in options.optprivs if "SystemRecoveryConfigPriv" in priv
                ) and "SystemRecoveryConfigPriv" not in list(self.getsesprivs().keys()):
                    raise IdTokenError(
                        "The currently logged in account must have The System "
                        "Recovery Config privilege to add the System Recovery "
                        "Config privilege."
                    )
                privs = self.getprivs(options)
                body["Oem"][self.rdmc.app.typepath.defs.oemhp]["Privileges"] = privs

            if options.role and self.rdmc.app.getiloversion() >= 5.140:
                body["RoleId"] = options.role

            if not body:
                raise InvalidCommandLineError(
                    "Valid Privileges/RoleID have not been provided; "
                    " no changes have been made to this account: %s\n" % options.identifier
                )
            self.rdmc.app.patch_handler(path, body)

        elif options.command.lower() == "delete":
            if not acct:
                raise InvalidCommandLineError("Unable to find the specified account.")
            self.rdmc.app.delete_handler(path)

        elif "cert" in options.command.lower():
            certpath = "/redfish/v1/AccountService/UserCertificateMapping/"
            privs = self.getsesprivs()
            if self.rdmc.app.typepath.defs.isgen9:
                IncompatibleiLOVersionError("This operation is only available on gen 10 " "and newer machines.")
            elif not privs["UserConfigPriv"]:
                raise IdTokenError(
                    "The currently logged in account must have The User "
                    "Config privilege to manage certificates for users."
                )
            else:
                if options.command.lower() == "addcert":
                    if not acct:
                        raise InvalidCommandLineError("Unable to find the specified account.")
                    body = {}
                    username = acct["UserName"]

                    fingerprintfile = options.certificate
                    if os.path.exists(fingerprintfile):
                        with open(fingerprintfile, "r") as fingerfile:
                            fingerprint = fingerfile.read()
                    else:
                        raise InvalidFileInputError("%s cannot be read." % fingerprintfile)
                    body = {"Fingerprint": fingerprint, "UserName": username}
                    self.rdmc.app.post_handler(certpath, body)

                elif options.command.lower() == "deletecert":
                    if not acct:
                        raise InvalidCommandLineError("Unable to find the specified account.")
                    certpath += acct["Id"]
                    self.rdmc.app.delete_handler(certpath)

        else:
            raise InvalidCommandLineError("Invalid command.")

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def getprivs(self, options):
        """find and return the privileges to set

        :param options: command line options
        :type options: list.
        """
        sesprivs = self.getsesprivs()
        setprivs = {}
        availableprivs = self.getsesprivs(availableprivsopts=True)

        if "UserConfigPriv" not in list(sesprivs.keys()):
            raise IdTokenError(
                "The currently logged in account does not have the User Config "
                "privilege and cannot add or modify user accounts."
            )

        if options.optprivs:
            for priv in options.optprivs:
                priv = next(iter(list(priv.keys())))
                if priv not in availableprivs:
                    raise IncompatibleiLOVersionError("Privilege %s is not available on this " "iLO version." % priv)

            if all(priv.values() for priv in options.optprivs):
                if (
                    any(priv for priv in options.optprivs if "SystemRecoveryConfigPriv" in priv)
                    and "SystemRecoveryConfigPriv" not in sesprivs.keys()
                ):
                    raise IdTokenError(
                        "The currently logged in account must have The System "
                        "Recovery Config privilege to add the System Recovery "
                        "Config privilege."
                    )
                else:
                    setprivs = {}
            for priv in options.optprivs:
                setprivs.update(priv)

        return setprivs

    def getsesprivs(self, availableprivsopts=False):
        """Finds and returns the curent session's privileges

        :param availableprivsopts: return available privileges
        :type availableprivsopts: boolean.
        """
        if self.rdmc.app.current_client:
            sespath = self.rdmc.app.current_client.session_location
            sespath = (
                self.rdmc.app.current_client.default_prefix
                + sespath.split(self.rdmc.app.current_client.default_prefix)[-1]
            )

            ses = self.rdmc.app.get_handler(sespath, service=False, silent=True)

            if not ses:
                raise SessionExpired("Invalid session. Please logout and " "log back in or include credentials.")

            sesprivs = {
                "HostBIOSConfigPriv": True,
                "HostNICConfigPriv": True,
                "HostStorageConfigPriv": True,
                "LoginPriv": True,
                "RemoteConsolePriv": True,
                "SystemRecoveryConfigPriv": True,
                "UserConfigPriv": True,
                "VirtualMediaPriv": True,
                "VirtualPowerAndResetPriv": True,
                "iLOConfigPriv": True,
            }
            if "Oem" in ses.dict:
                sesoemhp = ses.dict["Oem"][self.rdmc.app.typepath.defs.oemhp]
                if "Privileges" in list(sesoemhp.keys()):
                    sesprivs = sesoemhp["Privileges"]
            availableprivs = list(sesprivs.keys())
            updated_privs = dict()
            for priv, val in sesprivs.items():
                if val:
                    updated_privs[priv] = sesprivs[priv]
            sesprivs = updated_privs
            del updated_privs
        else:
            sesprivs = None
        if availableprivsopts:
            return availableprivs
        else:
            return sesprivs

    def credentialsvalidation(
        self,
        username="",
        loginname="",
        password="",
        acct=None,
        check_password=False,
        options=None,
    ):
        """sanity validation of credentials
        :param username: username to be added
        :type username: str.
        :param loginname: loginname to be added
        :type loginname: str.
        :param password: password to be added
        :type password: str.
        :param accounts: target federation account
        :type accounts: dict.
        :param check_password: flag to check password
        :type check_password: bool.
        """
        username_max_chars = 39  # 60
        loginname_max_chars = 39  # 60
        password_max_chars = 39  # PASSWORD MAX CHARS
        password_min_chars = 8  # PASSWORD MIN CHARS

        password_min_chars = next(iter(self.rdmc.app.select("AccountService."))).dict["Oem"][
            self.rdmc.app.typepath.defs.oemhp
        ]["MinPasswordLength"]

        if username != "" and loginname != "":
            if len(username) > username_max_chars:
                raise InvalidCommandLineError(
                    "Username exceeds maximum length" ". Use at most %s characters." % username_max_chars
                )

            if len(loginname) > loginname_max_chars:
                raise InvalidCommandLineError(
                    "Login name exceeds maximum " "length. Use at most %s characters." % loginname_max_chars
                )

            try:
                if acct:
                    if (
                        acct["UserName"] == username
                        or acct["Oem"][self.rdmc.app.typepath.defs.oemhp]["LoginName"] == loginname
                    ):
                        raise ResourceExists("Username or login name is already in use.")
            except KeyError:
                pass

        if check_password:
            if password == "" or password == "/r":
                raise InvalidCommandLineError("An invalid password was entered.")
            else:
                if len(password) > password_max_chars:
                    raise InvalidCommandLineError(
                        "Password length is invalid." " Use at most %s characters." % password_max_chars
                    )
                if len(password) < password_min_chars:
                    raise InvalidCommandLineError(
                        "Password length is invalid." " Use at least %s characters." % password_min_chars
                    )

    def iloaccountsvalidation(self, options):
        """add account validation function

        :param options: command line options
        :type options: list.
        """
        self.cmdbase.login_select_validation(self, options)

    @staticmethod
    def options_argument_group(parser):
        """Define optional arguments group

        :param parser: The parser to add the --addprivs option group to
        :type parser: ArgumentParser/OptionParser
        """
        group = parser.add_argument_group(
            "GLOBAL OPTIONS",
            "Options are available for all " "arguments within the scope of this command.",
        )

        group.add_argument(
            "--addprivs",
            dest="optprivs",
            nargs="*",
            action=_AccountParse,
            type=str,
            help="Optionally include this flag if you wish to specify "
            "which privileges you want added to the iLO account. Pick "
            "privileges from the privilege list in above help text. EX: --addprivs=1,2,4",
            default=None,
            metavar="Priv,",
        )

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)
        subcommand_parser = customparser.add_subparsers(dest="command")
        privilege_help = (
            "\n\n\tPRIVILEGES:\n\t1: Login\n\t2: Remote Console\n\t3: User Config\n\t4:"
            " iLO Config\n\t5: Virtual Media\n\t6: Virtual Power and Reset\n\n\tiLO 5 added "
            "privileges:\n\t7: Host NIC Config\n\t8: Host Bios Config\n\t9: Host Storage Config"
            "\n\t10: System Recovery Config"
        )
        # default sub-parser
        default_parser = subcommand_parser.add_parser(
            "default",
            help="Running without any sub-command will return all account information on the\n"
            "currently logged in server.",
        )
        default_parser.add_argument(
            "-j",
            "--json",
            dest="json",
            action="store_true",
            help="Optionally include this flag if you wish to change the"
            " displayed output to JSON format. Preserving the JSON data"
            " structure makes the information easier to parse.",
            default=False,
        )
        self.cmdbase.add_login_arguments_group(default_parser)
        # add sub-parser
        add_help = (
            "Adds an iLO user account to the currently logged in server with privileges\n" "specified in --addprivs."
        )
        add_parser = subcommand_parser.add_parser(
            __subparsers__[0],
            help=add_help,
            description=add_help + "\n\t*Note*:By default only the login privilege is added to the"
            ' newly created account\n\twith role "ReadOnly"in iLO 5 and no privileges in iLO 4.'
            + privilege_help
            + "\n\n\tExamples:\n\n\tAdd an account with specific privileges:\n\t\tiloaccounts add "
            "username accountname password --addprivs 1,2,4\n\n\tAdd an account and specify "
            "privileges by role:\n\t\tiloaccounts add username accountname password --role "
            "ReadOnly",
            formatter_class=RawDescriptionHelpFormatter,
        )
        # addprivs
        add_parser.add_argument(
            "identifier",
            help="The username or ID of the iLO account to modify.",
            metavar="USERNAMEorID#",
        )
        add_parser.add_argument(
            "loginname",
            help="The loginname of the iLO account to add. This is NOT used to login to the newly " "created account.",
            metavar="LOGINNAME",
        )
        add_parser.add_argument(
            "acct_password",
            help="The password of the iLO account to add. If you do not include a password, you "
            "will be prompted to enter one before an account is created. This is used to login to "
            "the newly created account.",
            metavar="PASSWORD",
            nargs="?",
            default="",
        )
        add_parser.add_argument(
            "--role",
            dest="role",
            choices=["Administrator", "ReadOnly", "Operator"],
            help="Optionally include this option if you would like to specify Privileges by role."
            " Roles are a set of privileges created based on the role of the account.",
            default=None,
        )
        add_parser.add_argument(
            "--serviceaccount",
            dest="serviceacc",
            action="store_true",
            help="Optionally include this flag if you wish to created account " "to be a service account.",
            default=False,
        )
        self.cmdbase.add_login_arguments_group(add_parser)

        self.options_argument_group(add_parser)
        # modify sub-parser
        modify_help = (
            "Modifies the provided iLO user account on the currently logged in server"
            '\nadding privileges using "--addprivs" to include privileges and using\n'
            '"--removeprivs" for removing privileges.'
        )
        modify_parser = subcommand_parser.add_parser(
            __subparsers__[1],
            help=modify_help,
            description=modify_help + privilege_help + "\n\n\tExamples:\n\n\tModify an iLO account's "
            "privileges by adding:\n\tiloaccounts modify username --addprivs 3,5\n\n\t"
            "Modify an iLO account's privileges by removal:\n\tiloaccounts modify username "
            "--removeprivs 10\n\n\tOr modify an iLO account's privileges by both simultaneously "
            "adding and removing privleges:\n\n\tiloaccounts modify username --addprivs 3,7 "
            "--removeprivs 9,10",
            formatter_class=RawDescriptionHelpFormatter,
        )
        modify_parser.add_argument(
            "identifier",
            help="The username or ID of the iLO account to modify.",
            metavar="USERNAMEorID#",
        )
        self.cmdbase.add_login_arguments_group(modify_parser)

        self.options_argument_group(modify_parser)  # addprivs
        modify_parser.add_argument(
            "--role",
            dest="role",
            choices=["Administrator", "ReadOnly", "Operator"],
            help="Optionally include this option if you would like to specify Privileges by role."
            " Roles are a set of privileges created based on the role of the account.",
            default=None,
        )
        modify_parser.add_argument(
            "--removeprivs",
            dest="optprivs",
            nargs="*",
            action=_AccountParse,
            type=str,
            help="Include this flag if you wish to specify "
            "which privileges you want removed from the iLO account. Pick "
            "privileges from the privilege list in the above help text. EX: --removeprivs=1,2,4",
            default=None,
            metavar="PRIV,",
        )
        # changepass sub-parser
        changepass_help = "Changes the password of the provided iLO user account on the currently " "logged in server."
        changepass_parser = subcommand_parser.add_parser(
            __subparsers__[2],
            help=changepass_help,
            description=changepass_help + "\n\nExamples:\n\nChange the password of an account:\n\t"
            "iloaccounts changepass 2 newpassword",
            formatter_class=RawDescriptionHelpFormatter,
        )
        changepass_parser.add_argument(
            "identifier",
            help="The username or ID of the iLO account to change the password for.",
            metavar="USERNAMEorID#",
        )
        changepass_parser.add_argument(
            "acct_password",
            help="The password to change the selected iLO account to. If you do not include a "
            "password, you will be prompted to enter one before an account is created. This is "
            "used to login to the newly created account.",
            metavar="PASSWORD",
            nargs="?",
            default="",
        )
        self.cmdbase.add_login_arguments_group(changepass_parser)

        # delete sub-parser
        delete_help = "Deletes the provided iLO user account on the currently logged in server."
        delete_parser = subcommand_parser.add_parser(
            __subparsers__[3],
            help=delete_help,
            description=delete_help + "\n\nExamples:\n\nDelete an iLO account:\n\t" "iloaccounts delete username",
            formatter_class=RawDescriptionHelpFormatter,
        )
        delete_parser.add_argument(
            "identifier",
            help="The username or ID of the iLO account to delete.",
            metavar="USERNAMEorID#",
        )
        self.cmdbase.add_login_arguments_group(delete_parser)

        # addcert sub-parser
        addcert_help = "Adds a certificate to the provided iLO user account on the currently logged" " in server."
        addcert_parser = subcommand_parser.add_parser(
            __subparsers__[4],
            help=addcert_help,
            description=addcert_help + r"\n\nExamples:\n\nAdd a user certificate to the provided "
            r"iLO account.\n\tiloaccounts addcert accountUserName C:\Users\user\cert.txt",
            formatter_class=RawDescriptionHelpFormatter,
        )
        addcert_parser.add_argument(
            "identifier",
            help="The username or ID of the iLO account to add a certificate to.",
            metavar="USERNAMEorID#",
        )
        addcert_parser.add_argument(
            "certificate",
            help="The certificate to add to the provided iLO account.",
            metavar="X.509CERTIFICATE",
        )
        self.cmdbase.add_login_arguments_group(addcert_parser)

        # deletecert sub-parser
        deletecert_help = "Deletes a certificate to the provided iLO user account on the currently " "logged in server."
        deletecert_parser = subcommand_parser.add_parser(
            __subparsers__[5],
            help=deletecert_help,
            description=deletecert_help + "\n\nExamples:\n\nDelete a user certificate from the "
            "provided iLO account.\n\tiloaccounts deletecert username",
            formatter_class=RawDescriptionHelpFormatter,
        )
        deletecert_parser.add_argument(
            "identifier",
            help="The username or ID of the iLO account to delete the certificate from.",
            metavar="USERNAMEorID#",
        )
        self.cmdbase.add_login_arguments_group(deletecert_parser)



###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" SigRecompute Command for rdmc """

try:
    from rdmc_helper import (
        IncompatibleiLOVersionError,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        IncompatibleiLOVersionError,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        ReturnCodes,
    )


class SigRecomputeCommand:
    """Recalculate the signature of the servers configuration"""

    def __init__(self):
        self.ident = {
            "name": "sigrecompute",
            "usage": None,
            "description": "Recalculate the signature on "
            "the computers configuration.\n\texample: sigrecompute\n\n"
            "\tNote: sigrecompute command is not available on Redfish systems.",
            "summary": "Command to recalculate the signature of the computer's " "configuration.",
            "aliases": [],
            "auxcommands": [],
        }

        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main sigrecompute function

        :param line: string of arguments passed in
        :type line: str.
        :param help_disp: display help flag
        :type line: bool.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        if args:
            raise InvalidCommandLineError("Sigrecompute command takes no arguments.\n")

        self.sigrecomputevalidation(options)
        path = self.rdmc.app.typepath.defs.systempath

        if self.rdmc.app.typepath.defs.flagforrest:
            body = {"Action": "ServerSigRecompute", "Target": "/Oem/Hp"}
            self.rdmc.app.post_handler(path, body)
        else:
            raise IncompatibleiLOVersionError("Sigrecompute action not available on redfish.\n")

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def sigrecomputevalidation(self, options):
        """sigrecomputevalidation method validation function

        :param options: command line options
        :type options: list.
        """
        self.cmdbase.login_select_validation(self, options)

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)



"""Module containing functional tests for the showdcpmmpendingconfig command"""
from __future__ import absolute_import

import sys

from rdmc_helper import ReturnCodes


class TestShowDcpmmPendingConfigCommand(object):
    """Class for functional tests for the showdcpmmpendingconfig command"""

    @staticmethod
    def test_show_invalid_flag(ilorest):
        """Test function call with invalid flag"""
        results = ilorest.run(['showdcpmmpendingconfig', '--bad-flag'])
        sys.stdout.write("results.stdout: ")
        sys.stdout.write(results.stdout)
        sys.stdout.write("results.stderr: ")
        sys.stdout.write(results.stderr)
        assert results.retcode == ReturnCodes.INVALID_COMMAND_LINE_ERROR

    @staticmethod
    def test_show_help_flag(ilorest):
        """Test function call with help flag"""
        results = ilorest.run(['showdcpmmpendingconfig', '-h'])
        sys.stdout.write("results.stdout: ")
        sys.stdout.write(results.stdout)
        sys.stdout.write("results.stderr: ")
        sys.stdout.write(results.stderr)
        assert results.retcode == ReturnCodes.SUCCESS

    @staticmethod
    def test_with_no_flag(ilorest, dl360_large):
        """Test for the case when `showdcpmmpendingconfig` command is run without any flag"""
        url, username, password = dl360_large.config
        with ilorest.session(url, username, password):
            results = ilorest.run(['showdcpmmpendingconfig'])
            sys.stdout.write("results.stdout: ")
            sys.stdout.write(results.stdout)
            sys.stdout.write("results.stderr: ")
            sys.stdout.write(results.stderr)
            assert results.retcode == ReturnCodes.SUCCESS

    @staticmethod
    def test_with_json_flag(ilorest, dl360_large):
        """Test for the case when `showdcpmmpendingconfig` command is run with json flag"""
        url, username, password = dl360_large.config
        with ilorest.session(url, username, password):
            results = ilorest.run(['showdcpmmpendingconfig', '-j'])
            sys.stdout.write("results.stdout: ")
            sys.stdout.write(results.stdout)
            sys.stdout.write("results.stderr: ")
            sys.stdout.write(results.stderr)
            assert results.retcode == ReturnCodes.SUCCESS

    @staticmethod
    def test_with_extraneous_attributes(ilorest):
        """
        Test for the case when `showdcpmmpendingconfig` command is run with extraneous attributes
        """
        results = ilorest.run(['showdcpmmpendingconfig', '1qazxcsd'])
        sys.stdout.write("results.stdout: ")
        sys.stdout.write(results.stdout)
        sys.stdout.write("results.stderr: ")
        sys.stdout.write(results.stderr)
        assert results.retcode == ReturnCodes.INVALID_COMMAND_LINE_ERROR



###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" Factory Defaults Command for rdmc """
import os
from argparse import RawDescriptionHelpFormatter

try:
    from rdmc_helper import (
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        InvalidFileInputError,
        InvalidPasswordLengthError,
        NoContentsFoundForOperationError,
        ReturnCodes,
        UploadError,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        ReturnCodes,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        NoContentsFoundForOperationError,
        InvalidFileInputError,
        InvalidPasswordLengthError,
        UploadError,
    )

from redfish.ris.ris import SessionExpired


class IloBackupRestoreCommand:
    """Backup and restore server using iLO's .bak file"""

    def __init__(self):
        self.ident = {
            "name": "backuprestore",
            "usage": None,
            "description": "Create a .bak file. \n\tExample: backuprestore backup\n\n\t"
            "Restore a server using a .bak file. \n\texample: backuprestore "
            "restore\n\n\tNOTE: This command is designed to only restore\n\tthe "
            "machine from which the backup file was created against.\n\tIf you would like to "
            "take one configuration and apply it\n\tto multiple systems see the "
            "serverclone command.\n\tThis command is only available in remote mode.",
            "summary": "Backup and restore iLO to a server using a .bak file.",
            "aliases": ["br"],
            "auxcommands": ["LogoutCommand"],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main factorydefaults function

        :param line: string of arguments passed in
        :type line: str.
        :param help_disp: display help flag
        :type line: bool.
        """
        if help_disp:
            line.append("-h")
            try:
                (_, _) = self.rdmc.rdmc_parse_arglist(self, line)
            except:
                return ReturnCodes.SUCCESS
            return ReturnCodes.SUCCESS
        try:
            (options, _) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        # if not len(args) == 1:
        #    raise InvalidCommandLineError("backuprestore command takes one argument.")

        self.ilobackuprestorevalidation(options)

        if "blobstore" in self.rdmc.app.current_client.base_url:
            raise InvalidCommandLineError("This command is only available remotely.")

        sessionkey = self.rdmc.app.current_client.session_key
        # sessionkey = (sessionkey).encode('ascii', 'ignore')

        if options.command == "backup":
            self.backupserver(options, sessionkey)
        elif options.command == "restore":
            self.restoreserver(options, sessionkey)
        else:
            raise InvalidCommandLineError("Only options are backup or restore\n")

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def backupserver(self, options, skey):
        """Create .bak file for a server

        :param options: command options
        :type options: list.
        :param skey: sessionkey of the currently logged in server
        :type skey: str.
        """
        select = "HpeiLOBackupRestoreService."
        backupfile = None
        results = self.rdmc.app.select(selector=select)

        try:
            results = results[0]
        except:
            pass

        if results:
            service = results.resp.dict
        else:
            raise NoContentsFoundForOperationError("%s not found.It may not " "be available on this system.\n" % select)

        backuplocation = service["BackupFileLocation"]
        backupname = backuplocation.split("/")[-1]

        postdata = []
        postdata.append(("sessionKey", skey))

        if options.fpass:
            if len(options.fpass) > 32:
                raise InvalidPasswordLengthError("Length of password cannot be greater than 32 characters.")
            postdata.append(("password", options.fpass))
        self.rdmc.ui.printer("Downloading backup file %s...\n" % backupname)
        backupfile = self.rdmc.app.post_handler(backuplocation, postdata, service=True, silent=True)

        if backupfile:
            if (backupfile.status != 200) and backupfile.read != "":
                if "Invalid Session" in backupfile.read:
                    raise SessionExpired("Invalid session. Please logout and log back in or include credentials.")
            self.rdmc.ui.printer("Download complete.\n")
            outfile = open(backupname, "wb")
            outfile.write(backupfile.ori)
            outfile.close()
        else:
            raise NoContentsFoundForOperationError("Unable to download file.\n")

    def restoreserver(self, options, skey):
        """Use a .bak file to restore a server

        :param options: command options
        :type options: list.
        :param skey: sessionkey of the currently logged in server
        :type skey: str.
        """

        select = "HpeiLOBackupRestoreService."

        if options.filename:
            filename = options.filename[0]
        else:
            files = []
            files = [f for f in os.listdir(".") if os.path.isfile(f) and f.endswith(".bak")]
            if files and len(files) > 1:
                raise InvalidFileInputError(
                    "More than one .bak file found in "
                    "the current directory. Please specify "
                    "a file using the -f option."
                )
            elif not files:
                raise InvalidFileInputError(
                    "No .bak file found in current " "directory. Please specify a file using the -f option."
                )
            else:
                filename = files[0]

        results = self.rdmc.app.select(selector=select)

        try:
            results = results[0]
        except:
            pass

        if results:
            service = results.resp.dict
        else:
            raise NoContentsFoundForOperationError("%s not found.It may not " "be available on this system." % select)
        restorelocation = service["HttpPushUri"]
        postdata = []

        with open(filename, "rb") as fle:
            bakfile = fle.read()
        postdata.append(("sessionKey", skey))
        if options.fpass:
            postdata.append(("password", options.fpass))
        postdata.append(("file", (filename, bakfile, "application/octet-stream")))
        if isinstance(skey, bytes):
            skey = skey.decode("utf-8")
        resp = self.rdmc.app.post_handler(
            restorelocation,
            postdata,
            service=False,
            silent=True,
            headers={"Cookie": "sessionKey=" + skey},
        )

        if not resp.status == 200:
            resp_ori = resp.ori
            if isinstance(resp_ori, bytes):
                resp_ori = resp_ori.decode("utf-8")
            if resp_ori == "invalid_restore_password":
                raise UploadError(
                    "Invalid or no password supplied during restore. Please "
                    "supply the password used during creation of the backup file."
                )
            else:
                raise UploadError("Error while uploading the backup file.")
        else:
            self.rdmc.ui.printer(
                "Restore in progress. iLO will be unresponsive while the "
                "restore completes.\nYour session will be terminated.\n"
            )
            self.auxcommands["logout"].run("")

    def ilobackuprestorevalidation(self, options):
        """factory defaults validation function

        :param options: command line options
        :type options: list.
        """
        self.cmdbase.login_select_validation(self, options)

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        subcommand_parser = customparser.add_subparsers(dest="command")
        subcommand_parser.required = True
        backup_help = "Create a backup of a server. This option is iLO5 Onwards"
        # backup sub-parser
        backup_parser = subcommand_parser.add_parser(
            "backup",
            help=backup_help,
            description=backup_help + "\n\texample: backuprestore backup " "--f <backup_file>  --filepass <password>",
            formatter_class=RawDescriptionHelpFormatter,
        )
        self.cmdbase.add_login_arguments_group(backup_parser)

        backup_parser.add_argument(
            "-f",
            "--filename",
            dest="filename",
            help="Use this flag to specify which backup file to restore. By "
            "default the commmand will try to find a .bak file in the current "
            "working directory.",
            action="append",
            default=None,
        )
        backup_parser.add_argument(
            "--filepass",
            dest="fpass",
            help="Optionally use the provided password when creating the "
            "backup file. The same password must be used for restoring.",
            default=None,
        )

        restore_help = "Restore a server with backup file. This option is for iLO5 Onwards"
        # backup sub-parser
        restore_parser = subcommand_parser.add_parser(
            "restore",
            help=restore_help,
            description=restore_help + "\n\texample: backuprestore restore " "--f <backup_file>  --filepass <password>",
            formatter_class=RawDescriptionHelpFormatter,
        )
        self.cmdbase.add_login_arguments_group(restore_parser)

        restore_parser.add_argument(
            "-f",
            "--filename",
            dest="filename",
            help="Use this flag to specify which backup file to restore. By "
            "default the commmand will try to find a .bak file in the current "
            "working directory.",
            action="append",
            default=None,
        )
        restore_parser.add_argument(
            "--filepass",
            dest="fpass",
            help="Optionally use the provided password when creating the "
            "backup file. The same password must be used for restoring.",
            default=None,
        )



"""
conftest.py is used to set up shared fixtures, plugins, hooks, other config etc.
This is the unit test conftest.py
http://docs.pytest.org/en/latest/writing_plugins.html#conftest-py-local-per-directory-plugins
"""
from __future__ import absolute_import

import pytest

from ..conftest import (
    ClearPendingConfigCommand,
    DisplayHelpers,
    MapperRenderers,
    RestHelpers,
    ShowPmemCommand,
)



"""Module to Test the 'cleardcpmmpendingconfig' command"""
from __future__ import absolute_import

import pytest

from rdmc_base_classes import RdmcCommandBase
from rdmc_helper import NoChangesFoundOrMadeError, NoContentsFoundForOperationError

from .conftest import ClearPendingConfigCommand, RestHelpers


class MockData(object):
    """
    Mock Data for 'cleardcpmmpendingconfig' command
    """
    task = None
    resp = None

    mock_tasks_1 = [{
        "@odata.context": "/redfish/v1/$metadata#Task.Task",
        "Name": "Task 1",
        "Payload": {
            "HttpOperation": "POST",
            "JsonBody": "{\"AddressRangeType\": \"PMEM\","
                        "\"InterleaveSets\": "
                        "[{\"Memory\": "
                        "{\"@odata.id\": \"/redfish/v1/Systems/1/Memory/proc1dimm1/\"}},"
                        "{\"Memory\": "
                        "{\"@odata.id\": \"/redfish/v1/Systems/1/Memory/proc1dimm12/\"}}],"
                        "\"Oem\": {\"Hpe\": {\"MemoryChunkSizePercentage\": 0}}}",
            "TargetUri": "/redfish/v1/Systems/1/MemoryDomains/PROC1MemoryDomain/"
                         "MemoryChunks/?$expand=."
        },
        "TaskState": "New"
    }, {
        "@odata.context": "/redfish/v1/$metadata#Task.Task",
        "Name": "Task 2",
        "Payload": {
            "HttpOperation": "POST",
            "JsonBody": "{\"AddressRangeType\": \"PMEM\","
                        "\"InterleaveSets\": "
                        "[{\"Memory\": "
                        "{\"@odata.id\": \"/redfish/v1/Systems/1/Memory/proc2dimm1/\"}},"
                        "{\"Memory\": "
                        "{\"@odata.id\": \"/redfish/v1/Systems/1/Memory/proc2dimm10/\"}}],"
                        "\"Oem\": {\"Hpe\": {\"MemoryChunkSizePercentage\": 0}}}",
            "TargetUri": "/redfish/v1/Systems/1/MemoryDomains/PROC2MemoryDomain/"
                         "MemoryChunks/?$expand=."
        },
        "TaskState": "Old"
    }]

    mock_tasks_2 = [{
        "@odata.context": "/redfish/v1/$metadata#Task.Task",
        "Name": "Task 1",
        "Payload": {
            "HttpOperation": "POST",
            "JsonBody": "{\"AddressRangeType\": \"PMEM\","
                        "\"InterleaveSets\": "
                        "[{\"Memory\": "
                        "{\"@odata.id\": \"/redfish/v1/Systems/1/Memory/proc1dimm1/\"}},"
                        "{\"Memory\": "
                        "{\"@odata.id\": \"/redfish/v1/Systems/1/Memory/proc1dimm12/\"}}],"
                        "\"Oem\": {\"Hpe\": {\"MemoryChunkSizePercentage\": 0}}}",
            "TargetUri": "/redfish/v1/Systems/1/MemoryDomains/PROC1MemoryDomain/"
                         "MemoryChunks/?$expand=."
        },
        "TaskState": "New"
    }]

    mock_tasks_3 = [{
        "@odata.context": "/redfish/v1/$metadata#Task.Task",
        "Name": "Task 1",
        "Payload": {
            "HttpOperation": "POST",
            "JsonBody": "{\"AddressRangeType\": \"PMEM\","
                        "\"InterleaveSets\": "
                        "[{\"Memory\": "
                        "{\"@odata.id\": \"/redfish/v1/Systems/1/Memory/proc1dimm1/\"}},"
                        "{\"Memory\": "
                        "{\"@odata.id\": \"/redfish/v1/Systems/1/Memory/proc1dimm12/\"}}],"
                        "\"Oem\": {\"Hpe\": {\"MemoryChunkSizePercentage\": 0}}}",
            "TargetUri": "/redfish/v1/Systems/1/MemoryDomains/PROC1MemoryDomain/?$expand=."
        },
        "TaskState": "New"
    }, {
        "@odata.context": "/redfish/v1/$metadata#Task.Task",
        "Name": "Task 2",
        "Payload": {
            "HttpOperation": "POST",
            "JsonBody": "{\"AddressRangeType\": \"PMEM\","
                        "\"InterleaveSets\": "
                        "[{\"Memory\": "
                        "{\"@odata.id\": \"/redfish/v1/Systems/1/Memory/proc2dimm1/\"}},"
                        "{\"Memory\": "
                        "{\"@odata.id\": \"/redfish/v1/Systems/1/Memory/proc2dimm10/\"}}],"
                        "\"Oem\": {\"Hpe\": {\"MemoryChunkSizePercentage\": 0}}}",
            "TargetUri": "/redfish/v1/Systems/1/MemoryDomains/PROC2MemoryDomain/?$expand=."
        },
        "TaskState": "Old"
    }]

    @staticmethod
    def get_resource_tasks(*argv):
        """
        Returns mock tasks
        """
        return MockData.task

    @staticmethod
    def get_response_code(*argv):
        """
        Returns mock response codes
        """
        return MockData.resp

class TestDeleteTasks(object):
    """
    Class for unit tests of 'cleardcpmmpendingconfig' command
    """

    @staticmethod
    @pytest.mark.parametrize("tasks, expected_result", [
        (MockData.mock_tasks_1, [MockData.mock_tasks_1[0]]),
        ([MockData.mock_tasks_2[0]], [MockData.mock_tasks_2[0]])
    ])
    def test_get_memory_chunk_tasks_pass(tasks, expected_result, monkeypatch):
        """
        Tests for 'get_memory_chunk_tasks' with valid data
        """
        MockData.task = tasks
        monkeypatch.setattr(RestHelpers.RestHelpers,
                            "retrieve_task_members", MockData.get_resource_tasks)
        pmem_command = ClearPendingConfigCommand.ClearPendingConfigCommand(RdmcCommandBase)
        result = pmem_command.get_memory_chunk_tasks()
        assert result == expected_result

    @staticmethod
    @pytest.mark.parametrize("tasks", [
        (MockData.mock_tasks_3),
        ([MockData.mock_tasks_3[0]]),
        ([MockData.mock_tasks_3[1]]),
        ([MockData.mock_tasks_1[1]])
    ])
    def test_get_memory_chunk_tasks_fail(tasks, monkeypatch):
        """
        Tests for 'get_memory_chunk_tasks' with invalid data
        """
        MockData.task = tasks
        monkeypatch.setattr(RestHelpers.RestHelpers, "retrieve_task_members",
                            MockData.get_resource_tasks)
        pmem_command = ClearPendingConfigCommand.ClearPendingConfigCommand(RdmcCommandBase)
        with pytest.raises(NoContentsFoundForOperationError):
            pmem_command.get_memory_chunk_tasks()

    @staticmethod
    @pytest.mark.parametrize("resp, verbose", [
        (200, False),
        (200, True)
    ])
    def test_delete_tasks_pass(resp, verbose, monkeypatch):
        """
        Tests for 'delete_tasks' with valid data
        """
        MockData.resp = resp
        monkeypatch.setattr(RestHelpers.RestHelpers, "delete_resource",
                            MockData.get_response_code)
        pmem_command = ClearPendingConfigCommand.ClearPendingConfigCommand(RdmcCommandBase)
        result = pmem_command.delete_tasks(MockData.mock_tasks_3, verbose)
        assert result is None

    @staticmethod
    @pytest.mark.parametrize("resp, verbose", [
        (None, False),
        (None, True)
    ])
    def test_delete_tasks_fail(resp, verbose, monkeypatch):
        """
        Tests for 'delete_tasks' with invalid data
        """
        MockData.resp = resp
        monkeypatch.setattr(RestHelpers.RestHelpers, "delete_resource",
                            MockData.get_response_code)
        pmem_command = ClearPendingConfigCommand.ClearPendingConfigCommand(RdmcCommandBase)
        with pytest.raises(NoChangesFoundOrMadeError):
            pmem_command.delete_tasks(MockData.mock_tasks_3, verbose)
